"""
Test breakpoint serialization.
"""

import os
import json
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class BreakpointSerialization(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @add_test_categories(["pyapi"])
    def test_resolvers(self):
        """Use Python APIs to test that we serialize resolvers."""
        self.build()
        self.setup_targets_and_cleanup()
        self.do_check_resolvers()

    def test_filters(self):
        """Use Python APIs to test that we serialize search filters correctly."""
        self.build()
        self.setup_targets_and_cleanup()
        self.do_check_filters()

    def test_options(self):
        """Use Python APIs to test that we serialize breakpoint options correctly."""
        self.build()
        self.setup_targets_and_cleanup()
        self.do_check_options()

    def test_appending(self):
        """Use Python APIs to test that we serialize breakpoint options correctly."""
        self.build()
        self.setup_targets_and_cleanup()
        self.do_check_appending()

    def test_name_filters(self):
        """Use python APIs to test that reading in by name works correctly."""
        self.build()
        self.setup_targets_and_cleanup()
        self.do_check_names()

    def test_scripted_extra_args(self):
        self.build()
        self.setup_targets_and_cleanup()
        self.do_check_extra_args()

    def test_resolver_serialization(self):
        """Test that breakpoint resolvers contain the expected information"""
        self.build()
        self.setup_targets_and_cleanup()

        exe_path = self.getBuildArtifact("a.out")
        exe_module = self.orig_target.module["a.out"]
        self.assertTrue(
            exe_module.IsValid(), "Failed to find the executable module in target"
        )
        sym_ctx_list = exe_module.FindFunctions("main")
        self.assertEqual(sym_ctx_list.GetSize(), 1, "Unable to find function 'main'")
        sym_ctx = sym_ctx_list.GetContextAtIndex(0)
        self.assertTrue(
            sym_ctx.IsValid(), "SBSymbolContext representing function 'main' is invalid"
        )
        main_func = sym_ctx.GetFunction()
        self.assertTrue(
            main_func.IsValid(), "SBFunction representing 'main' is invalid"
        )
        main_addr = main_func.GetStartAddress()

        bkpt = self.orig_target.BreakpointCreateBySBAddress(main_addr)
        self.assertTrue(
            bkpt.IsValid(), "Could not place breakpoint on 'main' by address"
        )
        stream = lldb.SBStream()
        sd = bkpt.SerializeToStructuredData()
        sd.GetAsJSON(stream)
        serialized_data = json.loads(stream.GetData())

        self.assertIn(
            exe_path,
            serialized_data["Breakpoint"]["BKPTResolver"]["Options"]["ModuleName"],
        )

    def test_structured_data_serialization(self):
        target = self.dbg.GetDummyTarget()
        self.assertTrue(target.IsValid(), VALID_TARGET)

        interpreter = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()
        interpreter.HandleCommand("br set -f foo -l 42", result)
        result = lldb.SBCommandReturnObject()
        interpreter.HandleCommand("br set -c 'argc == 1' -n main", result)

        bkp1 = target.GetBreakpointAtIndex(0)
        self.assertTrue(bkp1.IsValid(), VALID_BREAKPOINT)
        stream = lldb.SBStream()
        sd = bkp1.SerializeToStructuredData()
        sd.GetAsJSON(stream)
        serialized_data = json.loads(stream.GetData())
        self.assertEqual(
            serialized_data["Breakpoint"]["BKPTResolver"]["Options"]["FileName"], "foo"
        )
        self.assertEqual(
            serialized_data["Breakpoint"]["BKPTResolver"]["Options"]["LineNumber"], 42
        )

        bkp2 = target.GetBreakpointAtIndex(1)
        self.assertTrue(bkp2.IsValid(), VALID_BREAKPOINT)
        stream = lldb.SBStream()
        sd = bkp2.SerializeToStructuredData()
        sd.GetAsJSON(stream)
        serialized_data = json.loads(stream.GetData())
        self.assertIn(
            "main",
            serialized_data["Breakpoint"]["BKPTResolver"]["Options"]["SymbolNames"],
        )
        self.assertEqual(
            serialized_data["Breakpoint"]["BKPTOptions"]["ConditionText"], "argc == 1"
        )

        invalid_bkp = lldb.SBBreakpoint()
        self.assertFalse(invalid_bkp.IsValid(), "Breakpoint should not be valid.")
        stream = lldb.SBStream()
        sd = invalid_bkp.SerializeToStructuredData()
        sd.GetAsJSON(stream)
        self.assertFalse(
            stream.GetData(), "Invalid breakpoint should have an empty structured data"
        )

    def setup_targets_and_cleanup(self):
        def cleanup():
            self.RemoveTempFile(self.bkpts_file_path)

            if self.orig_target.IsValid():
                self.dbg.DeleteTarget(self.orig_target)
                self.dbg.DeleteTarget(self.copy_target)

        self.addTearDownHook(cleanup)
        self.RemoveTempFile(self.bkpts_file_path)

        exe = self.getBuildArtifact("a.out")

        # Create the targets we are making breakpoints in and copying them to:
        self.orig_target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.orig_target, VALID_TARGET)

        self.copy_target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.copy_target, VALID_TARGET)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        self.bkpts_file_path = self.getBuildArtifact("breakpoints.json")
        self.bkpts_file_spec = lldb.SBFileSpec(self.bkpts_file_path)

    def check_equivalence(self, source_bps, do_write=True):
        error = lldb.SBError()

        if do_write:
            error = self.orig_target.BreakpointsWriteToFile(
                self.bkpts_file_spec, source_bps
            )
            self.assertSuccess(error, "Failed writing breakpoints to file")

        copy_bps = lldb.SBBreakpointList(self.copy_target)
        error = self.copy_target.BreakpointsCreateFromFile(
            self.bkpts_file_spec, copy_bps
        )
        self.assertSuccess(error, "Failed reading breakpoints from file")

        num_source_bps = source_bps.GetSize()
        num_copy_bps = copy_bps.GetSize()
        self.assertEqual(
            num_source_bps,
            num_copy_bps,
            "Didn't get same number of input and output breakpoints - orig: %d copy: %d"
            % (num_source_bps, num_copy_bps),
        )

        for i in range(0, num_source_bps):
            source_bp = source_bps.GetBreakpointAtIndex(i)
            source_desc = lldb.SBStream()
            source_bp.GetDescription(source_desc, False)
            source_text = source_desc.GetData()

            # I am assuming here that the breakpoints will get written out in breakpoint ID order, and
            # read back in ditto.  That is true right now, and I can't see any reason to do it differently
            # but if we do we can go to writing the breakpoints one by one, or sniffing the descriptions to
            # see which one is which.
            copy_id = source_bp.GetID()
            copy_bp = copy_bps.FindBreakpointByID(copy_id)
            self.assertTrue(
                copy_bp.IsValid(), "Could not find copy breakpoint %d." % (copy_id)
            )

            copy_desc = lldb.SBStream()
            copy_bp.GetDescription(copy_desc, False)
            copy_text = copy_desc.GetData()

            # These two should be identical.
            self.trace("Source text for %d is %s." % (i, source_text))
            self.assertEqual(
                source_text,
                copy_text,
                "Source and dest breakpoints are not identical: \nsource: %s\ndest: %s"
                % (source_text, copy_text),
            )
            self.assertEqual(
                source_bp.GetNumLocations(),
                copy_bp.GetNumLocations(),
                "Source and dest num locations are not the same",
            )

    def do_check_resolvers(self):
        """Use Python APIs to check serialization of breakpoint resolvers"""

        empty_module_list = lldb.SBFileSpecList()
        empty_cu_list = lldb.SBFileSpecList()
        blubby_file_spec = lldb.SBFileSpec(
            os.path.join(self.getSourceDir(), "blubby.c")
        )

        # It isn't actually important for these purposes that these breakpoint
        # actually have locations.
        source_bps = lldb.SBBreakpointList(self.orig_target)
        source_bps.Append(self.orig_target.BreakpointCreateByLocation("blubby.c", 666))
        # Make sure we do one breakpoint right:
        self.check_equivalence(source_bps)
        source_bps.Clear()

        source_bps.Append(
            self.orig_target.BreakpointCreateByName(
                "blubby", lldb.eFunctionNameTypeAuto, empty_module_list, empty_cu_list
            )
        )
        source_bps.Append(
            self.orig_target.BreakpointCreateByName(
                "blubby", lldb.eFunctionNameTypeFull, empty_module_list, empty_cu_list
            )
        )
        source_bps.Append(
            self.orig_target.BreakpointCreateBySourceRegex(
                "dont really care", blubby_file_spec
            )
        )

        # And some number greater than one:
        self.check_equivalence(source_bps)

    def do_check_filters(self):
        """Use Python APIs to check serialization of breakpoint filters."""
        module_list = lldb.SBFileSpecList()
        module_list.Append(lldb.SBFileSpec("SomeBinary"))
        module_list.Append(lldb.SBFileSpec("SomeOtherBinary"))

        cu_list = lldb.SBFileSpecList()
        cu_list.Append(lldb.SBFileSpec("SomeCU.c"))
        cu_list.Append(lldb.SBFileSpec("AnotherCU.c"))
        cu_list.Append(lldb.SBFileSpec("ThirdCU.c"))

        blubby_file_spec = lldb.SBFileSpec(
            os.path.join(self.getSourceDir(), "blubby.c")
        )

        # It isn't actually important for these purposes that these breakpoint
        # actually have locations.
        source_bps = lldb.SBBreakpointList(self.orig_target)
        bkpt = self.orig_target.BreakpointCreateByLocation(
            blubby_file_spec, 666, 0, module_list
        )
        source_bps.Append(bkpt)

        # Make sure we do one right:
        self.check_equivalence(source_bps)
        source_bps.Clear()

        bkpt = self.orig_target.BreakpointCreateByName(
            "blubby", lldb.eFunctionNameTypeAuto, module_list, cu_list
        )
        source_bps.Append(bkpt)
        bkpt = self.orig_target.BreakpointCreateByName(
            "blubby", lldb.eFunctionNameTypeFull, module_list, cu_list
        )
        source_bps.Append(bkpt)
        bkpt = self.orig_target.BreakpointCreateBySourceRegex(
            "dont really care", blubby_file_spec
        )
        source_bps.Append(bkpt)

        # And some number greater than one:
        self.check_equivalence(source_bps)

    def do_check_options(self):
        """Use Python APIs to check serialization of breakpoint options."""

        empty_module_list = lldb.SBFileSpecList()
        empty_cu_list = lldb.SBFileSpecList()
        blubby_file_spec = lldb.SBFileSpec(
            os.path.join(self.getSourceDir(), "blubby.c")
        )

        # It isn't actually important for these purposes that these breakpoint
        # actually have locations.
        source_bps = lldb.SBBreakpointList(self.orig_target)

        bkpt = self.orig_target.BreakpointCreateByLocation(
            lldb.SBFileSpec("blubby.c"), 666, 333, 0, lldb.SBFileSpecList()
        )
        bkpt.SetEnabled(False)
        bkpt.SetOneShot(True)
        bkpt.SetThreadID(10)
        source_bps.Append(bkpt)

        # Make sure we get one right:
        self.check_equivalence(source_bps)
        source_bps.Clear()

        bkpt = self.orig_target.BreakpointCreateByName(
            "blubby", lldb.eFunctionNameTypeAuto, empty_module_list, empty_cu_list
        )
        bkpt.SetIgnoreCount(10)
        bkpt.SetThreadName("grubby")
        source_bps.Append(bkpt)

        bkpt = self.orig_target.BreakpointCreateByName(
            "blubby", lldb.eFunctionNameTypeAuto, empty_module_list, empty_cu_list
        )
        bkpt.SetCondition("gonna remove this")
        bkpt.SetCondition("")
        source_bps.Append(bkpt)

        bkpt = self.orig_target.BreakpointCreateByName(
            "blubby", lldb.eFunctionNameTypeFull, empty_module_list, empty_cu_list
        )
        bkpt.SetCondition("something != something_else")
        bkpt.SetQueueName("grubby")
        bkpt.AddName("FirstName")
        bkpt.AddName("SecondName")
        bkpt.SetScriptCallbackBody(
            '\tprint("I am a function that prints.")\n\tprint("I don\'t do anything else")\n'
        )
        source_bps.Append(bkpt)

        bkpt = self.orig_target.BreakpointCreateBySourceRegex(
            "dont really care", blubby_file_spec
        )
        cmd_list = lldb.SBStringList()
        cmd_list.AppendString("frame var")
        cmd_list.AppendString("thread backtrace")

        bkpt.SetCommandLineCommands(cmd_list)
        source_bps.Append(bkpt)

        self.check_equivalence(source_bps)

    def do_check_appending(self):
        """Use Python APIs to check appending to already serialized options."""

        empty_module_list = lldb.SBFileSpecList()
        empty_cu_list = lldb.SBFileSpecList()
        blubby_file_spec = lldb.SBFileSpec(
            os.path.join(self.getSourceDir(), "blubby.c")
        )

        # It isn't actually important for these purposes that these breakpoint
        # actually have locations.

        all_bps = lldb.SBBreakpointList(self.orig_target)
        source_bps = lldb.SBBreakpointList(self.orig_target)

        bkpt = self.orig_target.BreakpointCreateByLocation(
            lldb.SBFileSpec("blubby.c"), 666, 333, 0, lldb.SBFileSpecList()
        )
        bkpt.SetEnabled(False)
        bkpt.SetOneShot(True)
        bkpt.SetThreadID(10)
        source_bps.Append(bkpt)
        all_bps.Append(bkpt)

        error = lldb.SBError()
        error = self.orig_target.BreakpointsWriteToFile(
            self.bkpts_file_spec, source_bps
        )
        self.assertSuccess(error, "Failed writing breakpoints to file")

        source_bps.Clear()

        bkpt = self.orig_target.BreakpointCreateByName(
            "main", lldb.eFunctionNameTypeAuto, empty_module_list, empty_cu_list
        )
        bkpt.SetIgnoreCount(10)
        bkpt.SetThreadName("grubby")
        source_bps.Append(bkpt)
        all_bps.Append(bkpt)

        bkpt = self.orig_target.BreakpointCreateByName(
            "main", lldb.eFunctionNameTypeFull, empty_module_list, empty_cu_list
        )
        bkpt.SetCondition("something != something_else")
        bkpt.SetQueueName("grubby")
        bkpt.AddName("FirstName")
        bkpt.AddName("SecondName")

        source_bps.Append(bkpt)
        all_bps.Append(bkpt)

        error = self.orig_target.BreakpointsWriteToFile(
            self.bkpts_file_spec, source_bps, True
        )
        self.assertSuccess(error, "Failed appending breakpoints to file")

        self.check_equivalence(all_bps)

    def do_check_names(self):
        bkpt = self.orig_target.BreakpointCreateByLocation(
            lldb.SBFileSpec("blubby.c"), 666, 333, 0, lldb.SBFileSpecList()
        )
        good_bkpt_name = "GoodBreakpoint"
        write_bps = lldb.SBBreakpointList(self.orig_target)
        bkpt.AddName(good_bkpt_name)
        write_bps.Append(bkpt)

        error = lldb.SBError()
        error = self.orig_target.BreakpointsWriteToFile(self.bkpts_file_spec, write_bps)
        self.assertSuccess(error, "Failed writing breakpoints to file")

        copy_bps = lldb.SBBreakpointList(self.copy_target)
        names_list = lldb.SBStringList()
        names_list.AppendString("NoSuchName")

        error = self.copy_target.BreakpointsCreateFromFile(
            self.bkpts_file_spec, names_list, copy_bps
        )
        self.assertSuccess(error, "Failed reading breakpoints from file")
        self.assertEqual(
            copy_bps.GetSize(), 0, "Found breakpoints with a nonexistent name."
        )

        names_list.AppendString(good_bkpt_name)
        error = self.copy_target.BreakpointsCreateFromFile(
            self.bkpts_file_spec, names_list, copy_bps
        )
        self.assertSuccess(error, "Failed reading breakpoints from file")
        self.assertEqual(copy_bps.GetSize(), 1, "Found the matching breakpoint.")

    def do_check_extra_args(self):
        import side_effect

        interp = self.dbg.GetCommandInterpreter()
        error = lldb.SBError()

        script_name = os.path.join(self.getSourceDir(), "resolver.py")

        command = "command script import " + script_name
        result = lldb.SBCommandReturnObject()
        interp.HandleCommand(command, result)
        self.assertTrue(
            result.Succeeded(), "com scr imp failed: %s" % (result.GetError())
        )

        # First make sure a scripted breakpoint with no args works:
        bkpt = self.orig_target.BreakpointCreateFromScript(
            "resolver.Resolver",
            lldb.SBStructuredData(),
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertTrue(bkpt.IsValid(), "Bkpt is valid")
        write_bps = lldb.SBBreakpointList(self.orig_target)

        error = self.orig_target.BreakpointsWriteToFile(self.bkpts_file_spec, write_bps)
        self.assertSuccess(error, "Failed writing breakpoints")

        side_effect.g_extra_args = None
        copy_bps = lldb.SBBreakpointList(self.copy_target)
        error = self.copy_target.BreakpointsCreateFromFile(
            self.bkpts_file_spec, copy_bps
        )
        self.assertSuccess(error, "Failed reading breakpoints")

        self.assertEqual(copy_bps.GetSize(), 1, "Got one breakpoint from file.")
        no_keys = lldb.SBStringList()
        side_effect.g_extra_args.GetKeys(no_keys)
        self.assertEqual(no_keys.GetSize(), 0, "Should have no keys")

        self.orig_target.DeleteAllBreakpoints()
        self.copy_target.DeleteAllBreakpoints()

        # Now try one with extra args:

        extra_args = lldb.SBStructuredData()
        stream = lldb.SBStream()
        stream.Print('{"first_arg" : "first_value", "second_arg" : "second_value"}')
        extra_args.SetFromJSON(stream)
        self.assertTrue(extra_args.IsValid(), "SBStructuredData is valid.")

        bkpt = self.orig_target.BreakpointCreateFromScript(
            "resolver.Resolver",
            extra_args,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertTrue(bkpt.IsValid(), "Bkpt is valid")
        write_bps = lldb.SBBreakpointList(self.orig_target)

        error = self.orig_target.BreakpointsWriteToFile(self.bkpts_file_spec, write_bps)
        self.assertSuccess(error, "Failed writing breakpoints")

        orig_extra_args = side_effect.g_extra_args
        self.assertTrue(orig_extra_args.IsValid(), "Extra args originally valid")

        orig_keys = lldb.SBStringList()
        orig_extra_args.GetKeys(orig_keys)
        self.assertEqual(2, orig_keys.GetSize(), "Should have two keys")

        side_effect.g_extra_args = None

        copy_bps = lldb.SBBreakpointList(self.copy_target)
        error = self.copy_target.BreakpointsCreateFromFile(
            self.bkpts_file_spec, copy_bps
        )
        self.assertSuccess(error, "Failed reading breakpoints")

        self.assertEqual(copy_bps.GetSize(), 1, "Got one breakpoint from file.")

        copy_extra_args = side_effect.g_extra_args
        copy_keys = lldb.SBStringList()
        copy_extra_args.GetKeys(copy_keys)
        self.assertEqual(2, copy_keys.GetSize(), "Copy should have two keys")

        for idx in range(0, orig_keys.GetSize()):
            key = orig_keys.GetStringAtIndex(idx)
            copy_value = copy_extra_args.GetValueForKey(key).GetStringValue(100)

            if key == "first_arg":
                self.assertEqual(copy_value, "first_value")
            elif key == "second_arg":
                self.assertEqual(copy_value, "second_value")
            else:
                self.Fail("Unknown key: %s" % (key))



"""
Test lldb-dap repl mode detection
"""

import lldbdap_testcase
import dap_server
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestDAP_repl_mode_detection(lldbdap_testcase.DAPTestCaseBase):
    def assertEvaluate(self, expression, regex):
        self.assertRegex(
            self.dap_server.request_evaluate(expression, context="repl")["body"][
                "result"
            ],
            regex,
        )

    def test_completions(self):
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        source = "main.cpp"
        breakpoint1_line = line_number(source, "// breakpoint 1")
        breakpoint2_line = line_number(source, "// breakpoint 2")

        self.set_source_breakpoints(source, [breakpoint1_line, breakpoint2_line])

        # The result of the commands should return the empty string.
        self.assertEvaluate("`command regex user_command s/^$/platform/", r"^$")
        self.assertEvaluate("`command alias alias_command platform", r"^$")
        self.assertEvaluate(
            "`command alias alias_command_with_arg platform select --sysroot %1 remote-linux",
            r"^$",
        )

        self.continue_to_next_stop()
        self.assertEvaluate("user_command", "474747")
        self.assertEvaluate("alias_command", "474747")
        self.assertEvaluate("alias_command_with_arg", "474747")
        self.assertEvaluate("platform", "474747")

        self.continue_to_next_stop()
        platform_help_needle = "Commands to manage and create platforms"
        self.assertEvaluate("user_command", platform_help_needle)
        self.assertEvaluate("alias_command", platform_help_needle)
        self.assertEvaluate(
            "alias_command_with_arg " + self.getBuildDir(), "Platform: remote-linux"
        )
        self.assertEvaluate("platform", platform_help_needle)



"""
Frame provider that passes through all frames but prefixes their function names.

This exercises the provider's ability to consult its parent StackFrameList
when constructing each frame, verifying that the push/pop mechanism correctly
routes frame lookups to the parent list.
"""

import lldb
from lldb.plugins.scripted_process import ScriptedFrame
from lldb.plugins.scripted_frame_provider import ScriptedFrameProvider


class PrefixedFrame(ScriptedFrame):
    """A frame that wraps a real frame but prefixes the function name."""

    def __init__(self, thread, idx, function_name, prefix):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)

        self.idx = idx
        self.function_name = prefix + function_name

    def get_id(self):
        return self.idx

    def get_pc(self):
        return 0

    def get_function_name(self):
        return self.function_name

    def is_artificial(self):
        return False

    def is_hidden(self):
        return False

    def get_register_context(self):
        return None


class PrefixPassThroughProvider(ScriptedFrameProvider):
    """
    Provider that passes through every frame from its parent StackFrameList
    but adds a prefix to each function name.

    This verifies that the provider can freely access its input_frames
    (the parent list) without hitting circular dependencies or deadlocks.
    """

    PREFIX = "my_custom_"

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        return "Provider that prefixes all function names with 'my_custom_'"

    def get_frame_at_index(self, idx):
        if idx < len(self.input_frames):
            frame = self.input_frames[idx]
            function_name = frame.GetFunctionName()
            return PrefixedFrame(self.thread, idx, function_name, self.PREFIX)
        return None


class UpperCasePassThroughProvider(ScriptedFrameProvider):
    """
    Provider that passes through every frame from its parent StackFrameList
    but upper-cases each function name.

    When chained after PrefixPassThroughProvider, the result should be
    e.g. 'MY_CUSTOM_BAZ'.
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        return "Provider that upper-cases all function names"

    def get_frame_at_index(self, idx):
        if idx < len(self.input_frames):
            frame = self.input_frames[idx]
            function_name = frame.GetFunctionName()
            return PrefixedFrame(self.thread, idx, function_name.upper(), "")
        return None


class BtProviderStarProvider(ScriptedFrameProvider):
    """
    Provider that runs 'bt --provider *' from within get_frame_at_index
    to verify that re-entrant provider queries don't deadlock or crash.

    On the first call to get_frame_at_index, it runs the command and stores
    the output. It then passes through all frames with a 'reentrant_' prefix.
    """

    PREFIX = "reentrant_"

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)
        self.bt_provider_star_output = None

    @staticmethod
    def get_description():
        return "Provider that runs 'bt --provider *' during frame construction"

    def get_frame_at_index(self, idx):
        if idx >= len(self.input_frames):
            return None

        # On the first frame request, run 'bt --provider *' re-entrantly.
        if self.bt_provider_star_output is None:
            debugger = self.target.GetDebugger()
            ci = debugger.GetCommandInterpreter()
            result = lldb.SBCommandReturnObject()
            ci.HandleCommand("bt --provider '*'", result)
            self.bt_provider_star_output = result.GetOutput() or ""

        frame = self.input_frames[idx]
        function_name = frame.GetFunctionName()
        return PrefixedFrame(self.thread, idx, function_name, self.PREFIX)


class ValidatingPrefixProvider(ScriptedFrameProvider):
    """
    Provider that prefixes function names AND validates it receives the
    parent StackFrameList (not its own output).

    When constructing frame N (where N > 0), peeks at input_frames[N-1].
    If that younger frame's name already carries the prefix, the provider
    was incorrectly given its own output list — it flags this by prepending
    'danger_will_robinson_' to the function name.
    """

    PREFIX = "my_custom_"

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        return "Validating provider that detects wrong input list"

    def get_frame_at_index(self, idx):
        if idx >= len(self.input_frames):
            return None

        frame = self.input_frames[idx]
        function_name = frame.GetFunctionName()

        # For frames after the first, peek at the younger (already-provided)
        # frame in input_frames. If it already has our prefix, we were handed
        # our own output list instead of the parent list.
        if idx > 0:
            younger = self.input_frames[idx - 1]
            if younger.GetFunctionName().startswith(self.PREFIX):
                return PrefixedFrame(
                    self.thread, idx, function_name, "danger_will_robinson_"
                )

        return PrefixedFrame(self.thread, idx, function_name, self.PREFIX)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CommandOverrideCallback(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.line = line_number("main.c", "Hello world.")

    def test_command_override_callback(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        ci = self.dbg.GetCommandInterpreter()
        self.assertTrue(ci, VALID_COMMAND_INTERPRETER)

        command_arg = ""

        def foo(*command_args):
            nonlocal command_arg
            command_arg = command_args[0]

        self.assertTrue(ci.SetCommandOverrideCallback("breakpoint set", foo))
        self.expect("breakpoint set -n main")
        self.assertTrue(command_arg == "breakpoint")



"""
Test that we obey thread conditioned breakpoints.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


def using_current(test, thread, breakpoint):
    bp_id = breakpoint.GetID()
    test.runCmd("break modify -t current {0}".format(bp_id))


def set_thread_id(test, thread, breakpoint):
    id = thread.id
    breakpoint.SetThreadID(id)


def set_thread_name(test, thread, breakpoint):
    breakpoint.SetThreadName("main-thread")


class ThreadSpecificBreakTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @add_test_categories(["pyapi"])
    @expectedFailureAll(
        oslist=["ios", "watchos", "tvos", "bridgeos"],
        archs=["armv7", "armv7k"],
        bugnumber="rdar://problem/34563920",
    )  # armv7 ios problem - breakpoint with tid qualifier isn't working
    @skipIf(oslist=["windows"], archs=["aarch64"])  # Flaky on buildbot
    def test_thread_id(self):
        self.do_test(set_thread_id)

    @skipUnlessDarwin
    @expectedFailureAll(
        oslist=["ios", "watchos", "tvos", "bridgeos"],
        archs=["armv7", "armv7k"],
        bugnumber="rdar://problem/34563920",
    )  # armv7 ios problem - breakpoint with tid qualifier isn't working
    def test_thread_name(self):
        self.do_test(set_thread_name)

    @expectedFailureAll(
        oslist=["ios", "watchos", "tvos", "bridgeos"],
        archs=["armv7", "armv7k"],
        bugnumber="rdar://problem/34563920",
    )  # armv7 ios problem - breakpoint with tid qualifier isn't working
    @skipIf(oslist=["windows"], archs=["aarch64"])  # Flaky on buildbot
    def test_current_token(self):
        self.do_test(using_current)

    def do_test(self, setter_method):
        """Test that we obey thread conditioned breakpoints."""
        self.build()
        main_source_spec = lldb.SBFileSpec("main.cpp")
        (
            target,
            process,
            main_thread,
            main_breakpoint,
        ) = lldbutil.run_to_source_breakpoint(
            self, "Set main breakpoint here", main_source_spec
        )

        thread_breakpoint = target.BreakpointCreateBySourceRegex(
            "Set thread-specific breakpoint here", main_source_spec
        )
        self.assertGreater(
            thread_breakpoint.GetNumLocations(),
            0,
            "thread breakpoint has no locations associated with it.",
        )

        # Set the thread-specific breakpoint to stop only on the main thread
        # before the secondary thread has a chance to execute it.  The main
        # thread joins the secondary thread, and then the main thread will
        # execute the code at the breakpoint.  If the thread-specific
        # breakpoint works, the next stop will be on the main thread.
        setter_method(self, main_thread, thread_breakpoint)

        process.Continue()
        next_stop_state = process.GetState()
        self.assertEqual(
            next_stop_state,
            lldb.eStateStopped,
            "We should have stopped at the thread breakpoint.",
        )
        stopped_threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, thread_breakpoint
        )
        self.assertEqual(
            len(stopped_threads),
            1,
            "thread breakpoint stopped at unexpected number of threads",
        )
        self.assertEqual(
            stopped_threads[0].GetThreadID(),
            main_thread.GetThreadID(),
            "thread breakpoint stopped at the wrong thread",
        )



"""Test SBValue::GetValueDidChange"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ValueVarUpdateTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def test_with_process_launch_api(self):
        """Test SBValue::GetValueDidChange"""
        # Get the full path to our executable to be attached/debugged.
        exe = self.getBuildArtifact(self.testMethodName)
        d = {"EXE": exe}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        target = self.dbg.CreateTarget(exe)

        breakpoint = target.BreakpointCreateBySourceRegex(
            "break here", lldb.SBFileSpec("main.c")
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        i = self.frame().FindVariable("i")
        i_val = i.GetValueAsUnsigned(0)
        c = self.frame().FindVariable("c")

        # Update any values from the SBValue objects so we can ask them if they
        # changed after a continue
        i.GetValueDidChange()
        c.GetChildAtIndex(1).GetValueDidChange()
        c.GetChildAtIndex(0).GetChildAtIndex(0).GetValueDidChange()

        if self.TraceOn():
            self.runCmd("frame variable")

        self.runCmd("continue")

        if self.TraceOn():
            self.runCmd("frame variable")

        self.assertNotEqual(
            i_val, i.GetValueAsUnsigned(0), "GetValue() is saying a lie"
        )
        self.assertTrue(i.GetValueDidChange(), "GetValueDidChange() is saying a lie")

        # Check complex type
        self.assertTrue(
            c.GetChildAtIndex(0).GetChildAtIndex(0).GetValueDidChange()
            and not c.GetChildAtIndex(1).GetValueDidChange(),
            "GetValueDidChange() is saying a lie",
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self,
            "Break here",
            lldb.SBFileSpec("main.cpp"),
        )

        self.expect_expr("f.bar()", result_type="double", result_value="5")
        self.expect_expr("cf.bar()", result_type="int", result_value="2")
        self.expect_expr("vf.bar()", result_type="short", result_value="8")
        self.expect_expr(
            "cvf.bar()", result_type="const char *", result_summary='"volatile"'
        )



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects.transform import pdl as transform_pdl


def run(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            print("\nTEST:", f.__name__)
            f(module)
        print(module)
    return f


@run
def testTypes(module: Module):
    # CHECK-LABEL: TEST: testTypes
    # CHECK: !transform.any_op
    any_op = transform.AnyOpType.get()
    print(any_op)

    # CHECK: !transform.any_param
    any_param = transform.AnyParamType.get()
    print(any_param)

    # CHECK: !transform.any_value
    any_value = transform.AnyValueType.get()
    print(any_value)

    # CHECK: !transform.op<"foo.bar">
    # CHECK: foo.bar
    concrete_op = transform.OperationType.get("foo.bar")
    print(concrete_op)
    print(concrete_op.operation_name)

    # CHECK: !transform.param<i32>
    # CHECK: i32
    param = transform.ParamType.get(IntegerType.get_signless(32))
    print(param)
    print(param.type)


@run
def testSequenceOp(module: Module):
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [transform.AnyOpType.get()],
        transform.AnyOpType.get(),
    )
    with InsertionPoint(sequence.body):
        res = transform.CastOp(transform.AnyOpType.get(), sequence.bodyTarget)
        res2 = transform.cast(transform.any_op_t(), res.result)
        transform.YieldOp([res2])
    # CHECK-LABEL: TEST: testSequenceOp
    # CHECK: transform.sequence
    # CHECK: ^{{.*}}(%[[ARG0:.+]]: !transform.any_op):
    # CHECK:   %[[RES:.+]] = cast %[[ARG0]] : !transform.any_op to !transform.any_op
    # CHECK:   %[[RES2:.+]] = cast %[[RES]] : !transform.any_op to !transform.any_op
    # CHECK:   yield %[[RES2]] : !transform.any_op
    # CHECK: }


@run
def testSequenceOp(module: Module):
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [transform.AnyOpType.get()],
        transform.AnyOpType.get(),
    )
    with InsertionPoint(sequence.body):
        transform.YieldOp([sequence.bodyTarget])
    # CHECK-LABEL: TEST: testSequenceOp
    # CHECK: = transform.sequence -> !transform.any_op failures(propagate) {
    # CHECK: ^{{.*}}(%[[ARG0:.+]]: !transform.any_op):
    # CHECK:   yield %[[ARG0]] : !transform.any_op
    # CHECK: }


@run
def testNestedSequenceOp(module: Module):
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate, [], transform.AnyOpType.get()
    )
    with InsertionPoint(sequence.body):
        nested = transform.SequenceOp(
            transform.FailurePropagationMode.Propagate, [], sequence.bodyTarget
        )
        with InsertionPoint(nested.body):
            doubly_nested = transform.SequenceOp(
                transform.FailurePropagationMode.Propagate,
                [transform.AnyOpType.get()],
                nested.bodyTarget,
            )
            with InsertionPoint(doubly_nested.body):
                transform.YieldOp([doubly_nested.bodyTarget])
            transform.YieldOp()
        transform.YieldOp()
    # CHECK-LABEL: TEST: testNestedSequenceOp
    # CHECK: transform.sequence failures(propagate) {
    # CHECK: ^{{.*}}(%[[ARG0:.+]]: !transform.any_op):
    # CHECK:   sequence %[[ARG0]] : !transform.any_op failures(propagate) {
    # CHECK:   ^{{.*}}(%[[ARG1:.+]]: !transform.any_op):
    # CHECK:     = sequence %[[ARG1]] : !transform.any_op -> !transform.any_op failures(propagate) {
    # CHECK:     ^{{.*}}(%[[ARG2:.+]]: !transform.any_op):
    # CHECK:       yield %[[ARG2]] : !transform.any_op
    # CHECK:     }
    # CHECK:   }
    # CHECK: }


@run
def testSequenceOpWithExtras(module: Module):
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.AnyOpType.get(),
        [transform.AnyOpType.get(), transform.OperationType.get("foo.bar")],
    )
    with InsertionPoint(sequence.body):
        transform.YieldOp()
    # CHECK-LABEL: TEST: testSequenceOpWithExtras
    # CHECK: transform.sequence failures(propagate)
    # CHECK: ^{{.*}}(%{{.*}}: !transform.any_op, %{{.*}}: !transform.any_op, %{{.*}}: !transform.op<"foo.bar">):
    sequence = transform.sequence(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.AnyOpType.get(),
        [transform.AnyOpType.get(), transform.OperationType.get("foo.bar")],
    )
    with InsertionPoint(sequence.body):
        transform.yield_()
    # CHECK: transform.sequence failures(propagate)
    # CHECK: ^{{.*}}(%{{.*}}: !transform.any_op, %{{.*}}: !transform.any_op, %{{.*}}: !transform.op<"foo.bar">):


@run
def testNestedSequenceOpWithExtras(module: Module):
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.AnyOpType.get(),
        [transform.AnyOpType.get(), transform.OperationType.get("foo.bar")],
    )
    with InsertionPoint(sequence.body):
        nested = transform.SequenceOp(
            transform.FailurePropagationMode.Propagate,
            [],
            sequence.bodyTarget,
            sequence.bodyExtraArgs,
        )
        with InsertionPoint(nested.body):
            transform.YieldOp()
        transform.YieldOp()
    # CHECK-LABEL: TEST: testNestedSequenceOpWithExtras
    # CHECK: transform.sequence failures(propagate)
    # CHECK: ^{{.*}}(%[[ARG0:.*]]: !transform.any_op, %[[ARG1:.*]]: !transform.any_op, %[[ARG2:.*]]: !transform.op<"foo.bar">):
    # CHECK:   sequence %[[ARG0]], %[[ARG1]], %[[ARG2]] : (!transform.any_op, !transform.any_op, !transform.op<"foo.bar">)


@run
def testTransformPDLOps(module: Module):
    withPdl = transform_pdl.WithPDLPatternsOp(transform.AnyOpType.get())
    with InsertionPoint(withPdl.body):
        sequence = transform.SequenceOp(
            transform.FailurePropagationMode.Propagate,
            [transform.AnyOpType.get()],
            withPdl.bodyTarget,
        )
        with InsertionPoint(sequence.body):
            match = transform_pdl.PDLMatchOp(
                transform.AnyOpType.get(), sequence.bodyTarget, "pdl_matcher"
            )
            transform.YieldOp(match)
    # CHECK-LABEL: TEST: testTransformPDLOps
    # CHECK: transform.with_pdl_patterns {
    # CHECK: ^{{.*}}(%[[ARG0:.+]]: !transform.any_op):
    # CHECK:   = sequence %[[ARG0]] : !transform.any_op -> !transform.any_op failures(propagate) {
    # CHECK:   ^{{.*}}(%[[ARG1:.+]]: !transform.any_op):
    # CHECK:     %[[RES:.+]] = pdl_match @pdl_matcher in %[[ARG1]]
    # CHECK:     yield %[[RES]] : !transform.any_op
    # CHECK:   }
    # CHECK: }


@run
def testNamedSequenceOp(module: Module):
    module.operation.attributes["transform.with_named_sequence"] = UnitAttr.get()
    named_sequence = transform.NamedSequenceOp(
        "__transform_main",
        [transform.AnyOpType.get()],
        [transform.AnyOpType.get()],
        arg_attrs=[{"transform.consumed": UnitAttr.get()}],
    )
    with InsertionPoint(named_sequence.body):
        transform.YieldOp([named_sequence.bodyTarget])
    # CHECK-LABEL: TEST: testNamedSequenceOp
    # CHECK: module attributes {transform.with_named_sequence} {
    # CHECK:   transform.named_sequence @__transform_main(%[[ARG0:.+]]: !transform.any_op {transform.consumed}) -> !transform.any_op {
    # CHECK:     yield %[[ARG0]] : !transform.any_op
    named_sequence = transform.named_sequence(
        "other_seq",
        [transform.AnyOpType.get()],
        [transform.AnyOpType.get()],
        arg_attrs=[{"transform.consumed": UnitAttr.get()}],
    )
    with InsertionPoint(named_sequence.body):
        transform.yield_([named_sequence.bodyTarget])
    # CHECK:   transform.named_sequence @other_seq(%[[ARG1:.+]]: !transform.any_op {transform.consumed}) -> !transform.any_op {
    # CHECK:     yield %[[ARG1]] : !transform.any_op


@run
def testGetParentOp(module: Module):
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate, [], transform.AnyOpType.get()
    )
    with InsertionPoint(sequence.body):
        transform.GetParentOp(
            transform.AnyOpType.get(),
            sequence.bodyTarget,
            isolated_from_above=True,
            nth_parent=2,
        )
        transform.get_parent_op(
            transform.AnyOpType.get(),
            sequence.bodyTarget,
            isolated_from_above=True,
            nth_parent=2,
            allow_empty_results=True,
            op_name="func.func",
            deduplicate=True,
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: testGetParentOp
    # CHECK: transform.sequence
    # CHECK: ^{{.*}}(%[[ARG1:.+]]: !transform.any_op):
    # CHECK:   = get_parent_op %[[ARG1]] {isolated_from_above, nth_parent = 2 : i64}
    # CHECK:   = get_parent_op %[[ARG1]] {allow_empty_results, deduplicate, isolated_from_above, nth_parent = 2 : i64, op_name = "func.func"}


@run
def testMergeHandlesOp(module: Module):
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate, [], transform.AnyOpType.get()
    )
    with InsertionPoint(sequence.body):
        res = transform.MergeHandlesOp([sequence.bodyTarget])
        transform.merge_handles([res.result], deduplicate=True)
        transform.YieldOp()
    # CHECK-LABEL: TEST: testMergeHandlesOp
    # CHECK: transform.sequence
    # CHECK: ^{{.*}}(%[[ARG1:.+]]: !transform.any_op):
    # CHECK:  %[[RES1:.+]] = merge_handles %[[ARG1]] : !transform.any_op
    # CHECK:               = merge_handles deduplicate %[[RES1]] : !transform.any_op


@run
def testApplyPatternsOpCompact(module: Module):
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate, [], transform.AnyOpType.get()
    )
    with InsertionPoint(sequence.body):
        with InsertionPoint(transform.ApplyPatternsOp(sequence.bodyTarget).patterns):
            transform.ApplyCanonicalizationPatternsOp()
        with InsertionPoint(
            transform.apply_patterns(
                sequence.bodyTarget,
                apply_cse=True,
                max_iterations=3,
                max_num_rewrites=5,
            ).patterns
        ):
            transform.ApplyCanonicalizationPatternsOp()
        transform.YieldOp()
        # CHECK-LABEL: TEST: testApplyPatternsOpCompact
        # CHECK: apply_patterns to
        # CHECK: transform.apply_patterns.canonicalization
        # CHECK: } : !transform.any_op
        # CHECK: apply_patterns to
        # CHECK: transform.apply_patterns.canonicalization
        # CHECK: } {apply_cse, max_iterations = 3 : i64, max_num_rewrites = 5 : i64} : !transform.any_op


@run
def testApplyPatternsOpWithType(module: Module):
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("test.dummy"),
    )
    with InsertionPoint(sequence.body):
        with InsertionPoint(transform.ApplyPatternsOp(sequence.bodyTarget).patterns):
            transform.ApplyCanonicalizationPatternsOp()
        transform.YieldOp()
        # CHECK-LABEL: TEST: testApplyPatternsOp
        # CHECK: apply_patterns to
        # CHECK: transform.apply_patterns.canonicalization
        # CHECK: !transform.op<"test.dummy">


@run
def testReplicateOp(module: Module):
    with_pdl = transform_pdl.WithPDLPatternsOp(transform.AnyOpType.get())
    with InsertionPoint(with_pdl.body):
        sequence = transform.SequenceOp(
            transform.FailurePropagationMode.Propagate, [], with_pdl.bodyTarget
        )
        with InsertionPoint(sequence.body):
            m1 = transform_pdl.PDLMatchOp(
                transform.AnyOpType.get(), sequence.bodyTarget, "first"
            )
            m2 = transform_pdl.PDLMatchOp(
                transform.AnyOpType.get(), sequence.bodyTarget, "second"
            )
            transform.ReplicateOp(m1, [m2])
            transform.replicate(m1, [m2])
            transform.YieldOp()
    # CHECK-LABEL: TEST: testReplicateOp
    # CHECK: %[[FIRST:.+]] = pdl_match
    # CHECK: %[[SECOND:.+]] = pdl_match
    # CHECK: %{{.*}} = replicate num(%[[FIRST]]) %[[SECOND]]
    # CHECK: %{{.*}} = replicate num(%[[FIRST]]) %[[SECOND]]


# CHECK-LABEL: TEST: testApplyRegisteredPassOp
@run
def testApplyRegisteredPassOp(module: Module):
    # CHECK: transform.sequence
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate, [], transform.AnyOpType.get()
    )
    with InsertionPoint(sequence.body):
        # CHECK:   %{{.*}} = apply_registered_pass "canonicalize" to {{.*}} : (!transform.any_op) -> !transform.any_op
        mod = transform.ApplyRegisteredPassOp(
            transform.AnyOpType.get(), sequence.bodyTarget, "canonicalize"
        )
        # CHECK:   %{{.*}} = apply_registered_pass "canonicalize"
        # CHECK-SAME:    with options = {"top-down" = false}
        # CHECK-SAME:    to {{.*}} : (!transform.any_op) -> !transform.any_op
        mod = transform.ApplyRegisteredPassOp(
            transform.AnyOpType.get(),
            mod.result,
            "canonicalize",
            options={"top-down": BoolAttr.get(False)},
        )
        # CHECK:   %[[MAX_ITER:.+]] = transform.param.constant
        max_iter = transform.param_constant(
            transform.AnyParamType.get(),
            IntegerAttr.get(IntegerType.get_signless(64), 10),
        )
        # CHECK:   %[[MAX_REWRITE:.+]] = transform.param.constant
        max_rewrites = transform.param_constant(
            transform.AnyParamType.get(),
            IntegerAttr.get(IntegerType.get_signless(64), 1),
        )
        # CHECK:   %{{.*}} = apply_registered_pass "canonicalize"
        # NB: MLIR has sorted the dict lexicographically by key:
        # CHECK-SAME:    with options = {"max-iterations" = %[[MAX_ITER]],
        # CHECK-SAME:                    "max-rewrites" =  %[[MAX_REWRITE]],
        # CHECK-SAME:                    "test-convergence" = true,
        # CHECK-SAME:                    "top-down" = false}
        # CHECK-SAME:    to %{{.*}} : (!transform.any_op, !transform.any_param, !transform.any_param) -> !transform.any_op
        mod = transform.apply_registered_pass(
            transform.AnyOpType.get(),
            mod,
            "canonicalize",
            options={
                "top-down": BoolAttr.get(False),
                "max-iterations": max_iter,
                "test-convergence": True,
                "max-rewrites": max_rewrites,
            },
        )
        # CHECK:   %{{.*}} = apply_registered_pass "symbol-privatize"
        # CHECK-SAME:    with options = {"exclude" = ["a", "b"]}
        # CHECK-SAME:    to %{{.*}} : (!transform.any_op) -> !transform.any_op
        mod = transform.apply_registered_pass(
            transform.AnyOpType.get(),
            mod,
            "symbol-privatize",
            options={"exclude": ("a", "b")},
        )
        # CHECK:   %[[SYMBOL_A:.+]] = transform.param.constant
        symbol_a = transform.param_constant(
            transform.AnyParamType.get(), StringAttr.get("a")
        )
        # CHECK:   %[[SYMBOL_B:.+]] = transform.param.constant
        symbol_b = transform.param_constant(
            transform.AnyParamType.get(), StringAttr.get("b")
        )
        # CHECK:   %{{.*}} = apply_registered_pass "symbol-privatize"
        # CHECK-SAME:    with options = {"exclude" = [%[[SYMBOL_A]], %[[SYMBOL_B]]]}
        # CHECK-SAME:    to %{{.*}} : (!transform.any_op, !transform.any_param, !transform.any_param) -> !transform.any_op
        mod = transform.apply_registered_pass(
            transform.AnyOpType.get(),
            mod,
            "symbol-privatize",
            options={"exclude": (symbol_a, symbol_b)},
        )
        transform.YieldOp()


# CHECK-LABEL: TEST: testForeachOp
@run
def testForeachOp(module: Module):
    # CHECK: transform.sequence
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [transform.AnyOpType.get()],
        transform.AnyOpType.get(),
    )
    with InsertionPoint(sequence.body):
        # CHECK: {{.*}} = foreach %{{.*}} : !transform.any_op -> !transform.any_op
        foreach1 = transform.ForeachOp(
            (transform.AnyOpType.get(),), (sequence.bodyTarget,)
        )
        with InsertionPoint(foreach1.body):
            # CHECK: transform.yield {{.*}} : !transform.any_op
            transform.yield_(foreach1.bodyTargets)

        a_val = transform.get_operand(
            transform.AnyValueType.get(), foreach1.result, [0]
        )
        a_param = transform.param_constant(
            transform.AnyParamType.get(), StringAttr.get("a_param")
        )

        # CHECK: {{.*}} = foreach %{{.*}}, %{{.*}}, %{{.*}} : !transform.any_op, !transform.any_value, !transform.any_param -> !transform.any_value, !transform.any_param
        foreach2 = transform.foreach(
            (transform.AnyValueType.get(), transform.AnyParamType.get()),
            (sequence.bodyTarget, a_val, a_param),
        )
        with InsertionPoint(foreach2.owner.body):
            # CHECK: transform.yield {{.*}} : !transform.any_value, !transform.any_param
            transform.yield_(foreach2.owner.bodyTargets[1:3])

        another_param = transform.param_constant(
            transform.AnyParamType.get(), StringAttr.get("another_param")
        )
        params = transform.merge_handles([a_param, another_param])

        # CHECK: {{.*}} = foreach %{{.*}}, %{{.*}}, %{{.*}} with_zip_shortest : !transform.any_op, !transform.any_param, !transform.any_param -> !transform.any_op
        foreach3 = transform.foreach(
            (transform.AnyOpType.get(),),
            (foreach1.result, foreach2[1], params),
            with_zip_shortest=True,
        )
        with InsertionPoint(foreach3.owner.body):
            # CHECK: transform.yield {{.*}} : !transform.any_op
            transform.yield_((foreach3.owner.bodyTargets[0],))

        transform.yield_((foreach3,))



"""
Test that plugins that load commands work correctly.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class PluginCommandTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        TestBase.setUp(self)

    @skipIfNoSBHeaders
    @skipIfHostIncompatibleWithTarget
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24778")
    @no_debug_info_test
    def test_load_plugin(self):
        """Test that plugins that load commands work correctly."""
        self.generateSource("plugin.cpp")

        plugin_name = "plugin"
        if sys.platform.startswith("darwin"):
            plugin_lib_name = "lib%s.dylib" % plugin_name
        else:
            plugin_lib_name = "lib%s.so" % plugin_name

        # Invoke the library build rule.
        self.buildLibrary("plugin.cpp", plugin_name)

        retobj = lldb.SBCommandReturnObject()

        cinterpreter = self.dbg.GetCommandInterpreter()
        retval = cinterpreter.HandleCommand(
            "plugin load %s" % self.getBuildArtifact(plugin_lib_name), retobj
        )

        self.assertTrue(cinterpreter.UserCommandExists("plugin_loaded_command"))
        retobj.Clear()

        retval = self.dbg.GetCommandInterpreter().HandleCommand(
            "plugin_loaded_command child abc def ghi", retobj
        )

        if self.TraceOn():
            print(retobj.GetOutput())

        self.expect(retobj, substrs=["abc def ghi"], exe=False)

        retobj.Clear()

        # check that abbreviations work correctly in plugin commands.
        retval = self.dbg.GetCommandInterpreter().HandleCommand(
            "plugin_loaded_ ch abc def ghi", retobj
        )

        if self.TraceOn():
            print(retobj.GetOutput())

        self.expect(retobj, substrs=["abc def ghi"], exe=False)

    @no_debug_info_test
    def test_invalid_plugin_invocation(self):
        self.expect(
            "plugin load a b",
            error=True,
            startstr="error: 'plugin load' requires one argument",
        )
        self.expect(
            "plugin load",
            error=True,
            startstr="error: 'plugin load' requires one argument",
        )

    @no_debug_info_test
    def test_invalid_plugin_target(self):
        self.expect(
            "plugin load ThisIsNotAValidPluginName",
            error=True,
            startstr="error: no such file",
        )



import lldb

_map_capping_size = 255


class libcxx_hash_table_SynthProvider:
    def __init__(self, valobj, dict):
        self.valobj = valobj
        self.num_elements = None
        self.next_element = None
        self.bucket_count = None

    def update(self):
        logger = lldb.formatters.Logger.Logger()
        self.num_elements = None
        self.next_element = None
        self.bucket_count = None
        try:
            # unordered_map is made up of a hash_map, which has 4 pieces in it:
            #   bucket list :
            #      array of buckets
            #   p1 (pair):
            #      first - pointer to first loaded element
            #   p2 (pair):
            #      first - number of elements
            #      second - hash function
            #   p3 (pair):
            #      first - max_load_factor
            #      second - equality operator function
            #
            # For display, we actually don't need to go inside the buckets, since 'p1' has a way to iterate over all
            # the elements directly.
            #
            # We will calculate other values about the map because they will be useful for the summary.
            #
            table = self.valobj.GetChildMemberWithName("__table_")

            bl_ptr = table.GetChildMemberWithName(
                "__bucket_list_"
            ).GetChildMemberWithName("__ptr_")
            self.bucket_array_ptr = bl_ptr.GetChildMemberWithName(
                "__first_"
            ).GetValueAsUnsigned(0)
            self.bucket_count = (
                bl_ptr.GetChildMemberWithName("__second_")
                .GetChildMemberWithName("__data_")
                .GetChildMemberWithName("__first_")
                .GetValueAsUnsigned(0)
            )
            logger >> "Bucket count = %r" % self.bucket_count

            self.begin_ptr = (
                table.GetChildMemberWithName("__p1_")
                .GetChildMemberWithName("__first_")
                .GetChildMemberWithName("__next_")
            )

            self.num_elements = (
                table.GetChildMemberWithName("__p2_")
                .GetChildMemberWithName("__first_")
                .GetValueAsUnsigned(0)
            )
            self.max_load_factor = (
                table.GetChildMemberWithName("__p3_")
                .GetChildMemberWithName("__first_")
                .GetValueAsUnsigned(0)
            )
            logger >> "Num elements = %r" % self.num_elements

            # save the pointers as we get them
            #   -- don't access this first element if num_element==0!
            self.elements_cache = []
            if self.num_elements:
                self.next_element = self.begin_ptr
            else:
                self.next_element = None
        except Exception as e:
            logger >> "Caught exception: %r" % e
            pass

    def num_children(self):
        global _map_capping_size
        num_elements = self.num_elements
        if num_elements is not None:
            if num_elements > _map_capping_size:
                num_elements = _map_capping_size
        return num_elements

    def has_children(self):
        return True

    def get_child_index(self, name):
        logger = lldb.formatters.Logger.Logger()
        try:
            return int(name.lstrip("[").rstrip("]"))
        except:
            return -1

    def get_child_at_index(self, index):
        logger = lldb.formatters.Logger.Logger()
        logger >> "Retrieving child " + str(index)
        if index < 0:
            return None
        if index >= self.num_children():
            return None

        # extend
        logger >> " : cache size starts with %d elements" % len(self.elements_cache)
        while index >= len(self.elements_cache):
            # if we hit the end before we get the index, give up:
            if not self.next_element:
                logger >> " : hit end of list"
                return None

            node = self.next_element.Dereference()

            value = node.GetChildMemberWithName("__value_")
            hash_value = node.GetChildMemberWithName("__hash_").GetValueAsUnsigned()
            self.elements_cache.append((value, hash_value))

            self.next_element = node.GetChildMemberWithName("__next_")
            if not self.next_element.GetValueAsUnsigned(0):
                self.next_element = None

        # hit the index! so we have the value
        logger >> " : cache size ends with %d elements" % len(self.elements_cache)
        value, hash_value = self.elements_cache[index]
        return self.valobj.CreateValueFromData(
            "[%d] <hash %d>" % (index, hash_value), value.GetData(), value.GetType()
        )


def __lldb_init_module(debugger, dict):
    debugger.HandleCommand(
        'type synthetic add -l unordered_multi.libcxx_hash_table_SynthProvider -x "^(std::__1::)unordered_(multi)?(map|set)<.+> >$" -w libcxx'
    )



"""
Make sure 'frame var' using DIL parser/evaultor works for local variables.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil

import os
import shutil
import time


class TestFrameVarDILMemberOf(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_frame_var(self):
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.expect("settings set target.experimental.use-DIL true", substrs=[""])
        self.expect_var_path("s.x", value="1")
        self.expect_var_path("s.r", type="int &")
        self.expect_var_path("sr.x", value="1")
        self.expect_var_path("sr.r", type="int &")
        self.expect_var_path("sp->x", value="1")
        self.expect_var_path("sp->r", type="int &")

        self.expect(
            "frame variable 'sp->foo'",
            error=True,
            substrs=['"foo" is not a member of "(Sx *) sp"'],
        )

        self.expect(
            "frame variable 'sp.x'",
            error=True,
            substrs=[
                "member reference type 'Sx *' is a pointer; did you mean to use '->'"
            ],
        )

        # Test for record typedefs.
        self.expect_var_path("sa.x", value="3")
        self.expect_var_path("sa.y", value="'\\x04'")

        # Check that '.' is allowed in both simple and legacy modes
        frame = thread.GetFrameAtIndex(0)
        simple = frame.GetValueForVariablePath("s.x", lldb.eDILModeSimple)
        legacy = frame.GetValueForVariablePath("s.x", lldb.eDILModeLegacy)
        self.assertSuccess(simple.GetError())
        self.assertSuccess(legacy.GetError())

        # Check that '->' is not allowed in simple mode, but allowed in legacy mode
        simple = frame.GetValueForVariablePath("sp->x", lldb.eDILModeSimple)
        legacy = frame.GetValueForVariablePath("sp->x", lldb.eDILModeLegacy)
        self.assertFailure(simple.GetError())
        self.assertSuccess(legacy.GetError())



##===-- lldbutil.py ------------------------------------------*- Python -*-===##
##
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
##
##===----------------------------------------------------------------------===##

"""
This LLDB module contains miscellaneous utilities.
Some of the test suite takes advantage of the utility functions defined here.
They can also be useful for general purpose lldb scripting.
"""

import lldb
import os
import sys
import io

# ===================================================
# Utilities for locating/checking executable programs
# ===================================================


def is_exe(fpath):
    """Returns True if fpath is an executable."""
    return os.path.isfile(fpath) and os.access(fpath, os.X_OK)


def which(program):
    """Returns the full path to a program; None otherwise."""
    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file
    return None


# ===================================================
# Disassembly for an SBFunction or an SBSymbol object
# ===================================================


def disassemble(target, function_or_symbol):
    """Disassemble the function or symbol given a target.

    It returns the disassembly content in a string object.
    """
    buf = io.StringIO()
    insts = function_or_symbol.GetInstructions(target)
    for i in insts:
        print(i, file=buf)
    return buf.getvalue()


# ==========================================================
# Integer (byte size 1, 2, 4, and 8) to bytearray conversion
# ==========================================================


def int_to_bytearray(val, bytesize):
    """Utility function to convert an integer into a bytearray.

    It returns the bytearray in the little endian format.  It is easy to get the
    big endian format, just do ba.reverse() on the returned object.
    """
    import struct

    if bytesize == 1:
        return bytearray([val])

    # Little endian followed by a format character.
    template = "<%c"
    if bytesize == 2:
        fmt = template % "h"
    elif bytesize == 4:
        fmt = template % "i"
    elif bytesize == 4:
        fmt = template % "q"
    else:
        return None

    packed = struct.pack(fmt, val)
    return bytearray(ord(x) for x in packed)


def bytearray_to_int(bytes, bytesize):
    """Utility function to convert a bytearray into an integer.

    It interprets the bytearray in the little endian format. For a big endian
    bytearray, just do ba.reverse() on the object before passing it in.
    """
    import struct

    if bytesize == 1:
        return bytes[0]

    # Little endian followed by a format character.
    template = "<%c"
    if bytesize == 2:
        fmt = template % "h"
    elif bytesize == 4:
        fmt = template % "i"
    elif bytesize == 4:
        fmt = template % "q"
    else:
        return None

    unpacked = struct.unpack(fmt, str(bytes))
    return unpacked[0]


# ==============================================================
# Get the description of an lldb object or None if not available
# ==============================================================
def get_description(obj, option=None):
    """Calls lldb_obj.GetDescription() and returns a string, or None.

    For SBTarget, SBBreakpointLocation, and SBWatchpoint lldb objects, an extra
    option can be passed in to describe the detailed level of description
    desired:
        o lldb.eDescriptionLevelBrief
        o lldb.eDescriptionLevelFull
        o lldb.eDescriptionLevelVerbose
    """
    method = getattr(obj, "GetDescription")
    if not method:
        return None
    tuple = (lldb.SBTarget, lldb.SBBreakpointLocation, lldb.SBWatchpoint)
    if isinstance(obj, tuple):
        if option is None:
            option = lldb.eDescriptionLevelBrief

    stream = lldb.SBStream()
    if option is None:
        success = method(stream)
    else:
        success = method(stream, option)
    if not success:
        return None
    return stream.GetData()


# =================================================
# Convert some enum value to its string counterpart
# =================================================


def state_type_to_str(enum):
    """Returns the stateType string given an enum."""
    if enum == lldb.eStateInvalid:
        return "invalid"
    elif enum == lldb.eStateUnloaded:
        return "unloaded"
    elif enum == lldb.eStateConnected:
        return "connected"
    elif enum == lldb.eStateAttaching:
        return "attaching"
    elif enum == lldb.eStateLaunching:
        return "launching"
    elif enum == lldb.eStateStopped:
        return "stopped"
    elif enum == lldb.eStateRunning:
        return "running"
    elif enum == lldb.eStateStepping:
        return "stepping"
    elif enum == lldb.eStateCrashed:
        return "crashed"
    elif enum == lldb.eStateDetached:
        return "detached"
    elif enum == lldb.eStateExited:
        return "exited"
    elif enum == lldb.eStateSuspended:
        return "suspended"
    else:
        raise Exception("Unknown StateType enum")


def stop_reason_to_str(enum):
    """Returns the stopReason string given an enum."""
    if enum == lldb.eStopReasonInvalid:
        return "invalid"
    elif enum == lldb.eStopReasonNone:
        return "none"
    elif enum == lldb.eStopReasonTrace:
        return "trace"
    elif enum == lldb.eStopReasonBreakpoint:
        return "breakpoint"
    elif enum == lldb.eStopReasonWatchpoint:
        return "watchpoint"
    elif enum == lldb.eStopReasonSignal:
        return "signal"
    elif enum == lldb.eStopReasonException:
        return "exception"
    elif enum == lldb.eStopReasonPlanComplete:
        return "plancomplete"
    elif enum == lldb.eStopReasonThreadExiting:
        return "threadexiting"
    else:
        raise Exception("Unknown StopReason enum")


def symbol_type_to_str(enum):
    """Returns the symbolType string given an enum."""
    if enum == lldb.eSymbolTypeInvalid:
        return "invalid"
    elif enum == lldb.eSymbolTypeAbsolute:
        return "absolute"
    elif enum == lldb.eSymbolTypeCode:
        return "code"
    elif enum == lldb.eSymbolTypeData:
        return "data"
    elif enum == lldb.eSymbolTypeTrampoline:
        return "trampoline"
    elif enum == lldb.eSymbolTypeRuntime:
        return "runtime"
    elif enum == lldb.eSymbolTypeException:
        return "exception"
    elif enum == lldb.eSymbolTypeSourceFile:
        return "sourcefile"
    elif enum == lldb.eSymbolTypeHeaderFile:
        return "headerfile"
    elif enum == lldb.eSymbolTypeObjectFile:
        return "objectfile"
    elif enum == lldb.eSymbolTypeCommonBlock:
        return "commonblock"
    elif enum == lldb.eSymbolTypeBlock:
        return "block"
    elif enum == lldb.eSymbolTypeLocal:
        return "local"
    elif enum == lldb.eSymbolTypeParam:
        return "param"
    elif enum == lldb.eSymbolTypeVariable:
        return "variable"
    elif enum == lldb.eSymbolTypeVariableType:
        return "variabletype"
    elif enum == lldb.eSymbolTypeLineEntry:
        return "lineentry"
    elif enum == lldb.eSymbolTypeLineHeader:
        return "lineheader"
    elif enum == lldb.eSymbolTypeScopeBegin:
        return "scopebegin"
    elif enum == lldb.eSymbolTypeScopeEnd:
        return "scopeend"
    elif enum == lldb.eSymbolTypeAdditional:
        return "additional"
    elif enum == lldb.eSymbolTypeCompiler:
        return "compiler"
    elif enum == lldb.eSymbolTypeInstrumentation:
        return "instrumentation"
    elif enum == lldb.eSymbolTypeUndefined:
        return "undefined"


def value_type_to_str(enum):
    """Returns the valueType string given an enum."""
    if enum == lldb.eValueTypeInvalid:
        return "invalid"
    elif enum == lldb.eValueTypeVariableGlobal:
        return "global_variable"
    elif enum == lldb.eValueTypeVariableStatic:
        return "static_variable"
    elif enum == lldb.eValueTypeVariableArgument:
        return "argument_variable"
    elif enum == lldb.eValueTypeVariableLocal:
        return "local_variable"
    elif enum == lldb.eValueTypeRegister:
        return "register"
    elif enum == lldb.eValueTypeRegisterSet:
        return "register_set"
    elif enum == lldb.eValueTypeConstResult:
        return "constant_result"
    else:
        raise Exception("Unknown ValueType enum")


# ==================================================
# Get stopped threads due to each stop reason.
# ==================================================


def sort_stopped_threads(
    process,
    breakpoint_threads=None,
    crashed_threads=None,
    watchpoint_threads=None,
    signal_threads=None,
    exiting_threads=None,
    other_threads=None,
):
    """Fills array *_threads with threads stopped for the corresponding stop
    reason.
    """
    for lst in [
        breakpoint_threads,
        watchpoint_threads,
        signal_threads,
        exiting_threads,
        other_threads,
    ]:
        if lst is not None:
            lst[:] = []

    for thread in process:
        dispatched = False
        for reason, list in [
            (lldb.eStopReasonBreakpoint, breakpoint_threads),
            (lldb.eStopReasonException, crashed_threads),
            (lldb.eStopReasonWatchpoint, watchpoint_threads),
            (lldb.eStopReasonSignal, signal_threads),
            (lldb.eStopReasonThreadExiting, exiting_threads),
            (None, other_threads),
        ]:
            if not dispatched and list is not None:
                if thread.GetStopReason() == reason or reason is None:
                    list.append(thread)
                    dispatched = True


# ==================================================
# Utility functions for setting breakpoints
# ==================================================


def run_break_set_by_file_and_line(
    test,
    file_name,
    line_number,
    extra_options=None,
    num_expected_locations=1,
    loc_exact=False,
    module_name=None,
):
    """Set a breakpoint by file and line, returning the breakpoint number.

    If extra_options is not None, then we append it to the breakpoint set command.

    If num_expected_locations is -1 we check that we got AT LEAST one location, otherwise we check that num_expected_locations equals the number of locations.

    If loc_exact is true, we check that there is one location, and that location must be at the input file and line number.
    """

    if file_name is None:
        command = "breakpoint set -l %d" % (line_number)
    else:
        command = 'breakpoint set -f "%s" -l %d' % (file_name, line_number)

    if module_name:
        command += " --shlib '%s'" % (module_name)

    if extra_options:
        command += " " + extra_options

    break_results = run_break_set_command(test, command)

    if num_expected_locations == 1 and loc_exact:
        check_breakpoint_result(
            test,
            break_results,
            num_locations=num_expected_locations,
            file_name=file_name,
            line_number=line_number,
            module_name=module_name,
        )
    else:
        check_breakpoint_result(
            test, break_results, num_locations=num_expected_locations
        )

    return get_bpno_from_match(break_results)


def run_break_set_by_symbol(
    test,
    symbol,
    extra_options=None,
    num_expected_locations=-1,
    sym_exact=False,
    module_name=None,
):
    """Set a breakpoint by symbol name.  Common options are the same as run_break_set_by_file_and_line.

    If sym_exact is true, then the output symbol must match the input exactly, otherwise we do a substring match.
    """
    command = 'breakpoint set -n "%s"' % (symbol)

    if module_name:
        command += " --shlib '%s'" % (module_name)

    if extra_options:
        command += " " + extra_options

    break_results = run_break_set_command(test, command)

    if num_expected_locations == 1 and sym_exact:
        check_breakpoint_result(
            test,
            break_results,
            num_locations=num_expected_locations,
            symbol_name=symbol,
            module_name=module_name,
        )
    else:
        check_breakpoint_result(
            test, break_results, num_locations=num_expected_locations
        )

    return get_bpno_from_match(break_results)


def run_break_set_by_selector(
    test, selector, extra_options=None, num_expected_locations=-1, module_name=None
):
    """Set a breakpoint by selector.  Common options are the same as run_break_set_by_file_and_line."""

    command = 'breakpoint set -S "%s"' % (selector)

    if module_name:
        command += ' --shlib "%s"' % (module_name)

    if extra_options:
        command += " " + extra_options

    break_results = run_break_set_command(test, command)

    if num_expected_locations == 1:
        check_breakpoint_result(
            test,
            break_results,
            num_locations=num_expected_locations,
            symbol_name=selector,
            symbol_match_exact=False,
            module_name=module_name,
        )
    else:
        check_breakpoint_result(
            test, break_results, num_locations=num_expected_locations
        )

    return get_bpno_from_match(break_results)


def run_break_set_by_regexp(
    test, regexp, extra_options=None, num_expected_locations=-1
):
    """Set a breakpoint by regular expression match on symbol name.  Common options are the same as run_break_set_by_file_and_line."""

    command = 'breakpoint set -r "%s"' % (regexp)
    if extra_options:
        command += " " + extra_options

    break_results = run_break_set_command(test, command)

    check_breakpoint_result(test, break_results, num_locations=num_expected_locations)

    return get_bpno_from_match(break_results)


def run_break_set_by_source_regexp(
    test, regexp, extra_options=None, num_expected_locations=-1
):
    """Set a breakpoint by source regular expression.  Common options are the same as run_break_set_by_file_and_line."""
    command = 'breakpoint set -p "%s"' % (regexp)
    if extra_options:
        command += " " + extra_options

    break_results = run_break_set_command(test, command)

    check_breakpoint_result(test, break_results, num_locations=num_expected_locations)

    return get_bpno_from_match(break_results)


def run_break_set_command(test, command):
    """Run the command passed in - it must be some break set variant - and analyze the result.
    Returns a dictionary of information gleaned from the command-line results.
    Will assert if the breakpoint setting fails altogether.

    Dictionary will contain:
        bpno          - breakpoint of the newly created breakpoint, -1 on error.
        num_locations - number of locations set for the breakpoint.

    If there is only one location, the dictionary MAY contain:
        file          - source file name
        line_no       - source line number
        symbol        - symbol name
        inline_symbol - inlined symbol name
        offset        - offset from the original symbol
        module        - module
        address       - address at which the breakpoint was set."""

    patterns = [
        r"^Breakpoint (?P<bpno>[0-9]+): (?P<num_locations>[0-9]+) locations\.$",
        r"^Breakpoint (?P<bpno>[0-9]+): (?P<num_locations>no) locations \(pending\)\.",
        r"^Breakpoint (?P<bpno>[0-9]+): where = (?P<module>.*)`(?P<symbol>[+\-]{0,1}[^+]+)( \+ (?P<offset>[0-9]+)){0,1}( \[inlined\] (?P<inline_symbol>.*)){0,1} at (?P<file>[^:]+):(?P<line_no>[0-9]+), address = (?P<address>0x[0-9a-fA-F]+)$",
        r"^Breakpoint (?P<bpno>[0-9]+): where = (?P<module>.*)`(?P<symbol>.*)( \+ (?P<offset>[0-9]+)){0,1}, address = (?P<address>0x[0-9a-fA-F]+)$",
    ]
    match_object = test.match(command, patterns)
    break_results = match_object.groupdict()

    # We always insert the breakpoint number, setting it to -1 if we couldn't find it
    # Also, make sure it gets stored as an integer.
    if not "bpno" in break_results:
        break_results["bpno"] = -1
    else:
        break_results["bpno"] = int(break_results["bpno"])

    # We always insert the number of locations
    # If ONE location is set for the breakpoint, then the output doesn't mention locations, but it has to be 1...
    # We also make sure it is an integer.

    if not "num_locations" in break_results:
        num_locations = 1
    else:
        num_locations = break_results["num_locations"]
        if num_locations == "no":
            num_locations = 0
        else:
            num_locations = int(break_results["num_locations"])

    break_results["num_locations"] = num_locations

    if "line_no" in break_results:
        break_results["line_no"] = int(break_results["line_no"])

    return break_results


def get_bpno_from_match(break_results):
    return int(break_results["bpno"])


def check_breakpoint_result(
    test,
    break_results,
    file_name=None,
    line_number=-1,
    symbol_name=None,
    symbol_match_exact=True,
    module_name=None,
    offset=-1,
    num_locations=-1,
):
    out_num_locations = break_results["num_locations"]

    if num_locations == -1:
        test.assertTrue(
            out_num_locations > 0, "Expecting one or more locations, got none."
        )
    else:
        test.assertTrue(
            num_locations == out_num_locations,
            "Expecting %d locations, got %d." % (num_locations, out_num_locations),
        )

    if file_name:
        out_file_name = ""
        if "file" in break_results:
            out_file_name = break_results["file"]
        test.assertTrue(
            file_name == out_file_name,
            "Breakpoint file name '%s' doesn't match resultant name '%s'."
            % (file_name, out_file_name),
        )

    if line_number != -1:
        out_file_line = -1
        if "line_no" in break_results:
            out_line_number = break_results["line_no"]

        test.assertTrue(
            line_number == out_line_number,
            "Breakpoint line number %s doesn't match resultant line %s."
            % (line_number, out_line_number),
        )

    if symbol_name:
        out_symbol_name = ""
        # Look first for the inlined symbol name, otherwise use the symbol
        # name:
        if "inline_symbol" in break_results and break_results["inline_symbol"]:
            out_symbol_name = break_results["inline_symbol"]
        elif "symbol" in break_results:
            out_symbol_name = break_results["symbol"]

        if symbol_match_exact:
            test.assertTrue(
                symbol_name == out_symbol_name,
                "Symbol name '%s' doesn't match resultant symbol '%s'."
                % (symbol_name, out_symbol_name),
            )
        else:
            test.assertTrue(
                out_symbol_name.find(symbol_name) != -1,
                "Symbol name '%s' isn't in resultant symbol '%s'."
                % (symbol_name, out_symbol_name),
            )

    if module_name:
        out_nodule_name = None
        if "module" in break_results:
            out_module_name = break_results["module"]

        test.assertTrue(
            module_name.find(out_module_name) != -1,
            "Symbol module name '%s' isn't in expected module name '%s'."
            % (out_module_name, module_name),
        )


# ==================================================
# Utility functions related to Threads and Processes
# ==================================================


def get_stopped_threads(process, reason):
    """Returns the thread(s) with the specified stop reason in a list.

    The list can be empty if no such thread exists.
    """
    threads = []
    for t in process:
        if t.GetStopReason() == reason:
            threads.append(t)
    return threads


def get_stopped_thread(process, reason):
    """A convenience function which returns the first thread with the given stop
    reason or None.

    Example usages:

    1. Get the stopped thread due to a breakpoint condition

    ...
        from lldbutil import get_stopped_thread
        thread = get_stopped_thread(process, lldb.eStopReasonPlanComplete)
        self.assertTrue(thread.IsValid(), "There should be a thread stopped due to breakpoint condition")
    ...

    2. Get the thread stopped due to a breakpoint

    ...
        from lldbutil import get_stopped_thread
        thread = get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertTrue(thread.IsValid(), "There should be a thread stopped due to breakpoint")
    ...

    """
    threads = get_stopped_threads(process, reason)
    if len(threads) == 0:
        return None
    return threads[0]


def get_threads_stopped_at_breakpoint(process, bkpt):
    """For a stopped process returns the thread stopped at the breakpoint passed in bkpt"""
    stopped_threads = []
    threads = []

    stopped_threads = get_stopped_threads(process, lldb.eStopReasonBreakpoint)

    if len(stopped_threads) == 0:
        return threads

    for thread in stopped_threads:
        # Make sure we've hit our breakpoint...
        break_id = thread.GetStopReasonDataAtIndex(0)
        if break_id == bkpt.GetID():
            threads.append(thread)

    return threads


def continue_to_breakpoint(process, bkpt):
    """Continues the process, if it stops, returns the threads stopped at bkpt; otherwise, returns None"""
    process.Continue()
    if process.GetState() != lldb.eStateStopped:
        return None
    else:
        return get_threads_stopped_at_breakpoint(process, bkpt)


def get_caller_symbol(thread):
    """
    Returns the symbol name for the call site of the leaf function.
    """
    depth = thread.GetNumFrames()
    if depth <= 1:
        return None
    caller = thread.GetFrameAtIndex(1).GetSymbol()
    if caller:
        return caller.GetName()
    else:
        return None


def get_function_names(thread):
    """
    Returns a sequence of function names from the stack frames of this thread.
    """

    def GetFuncName(i):
        return thread.GetFrameAtIndex(i).GetFunctionName()

    return [GetFuncName(i) for i in range(thread.GetNumFrames())]


def get_symbol_names(thread):
    """
    Returns a sequence of symbols for this thread.
    """

    def GetSymbol(i):
        return thread.GetFrameAtIndex(i).GetSymbol().GetName()

    return [GetSymbol(i) for i in range(thread.GetNumFrames())]


def get_pc_addresses(thread):
    """
    Returns a sequence of pc addresses for this thread.
    """

    def GetPCAddress(i):
        return thread.GetFrameAtIndex(i).GetPCAddress()

    return [GetPCAddress(i) for i in range(thread.GetNumFrames())]


def get_filenames(thread):
    """
    Returns a sequence of file names from the stack frames of this thread.
    """

    def GetFilename(i):
        return thread.GetFrameAtIndex(i).GetLineEntry().GetFileSpec().GetFilename()

    return [GetFilename(i) for i in range(thread.GetNumFrames())]


def get_line_numbers(thread):
    """
    Returns a sequence of line numbers from the stack frames of this thread.
    """

    def GetLineNumber(i):
        return thread.GetFrameAtIndex(i).GetLineEntry().GetLine()

    return [GetLineNumber(i) for i in range(thread.GetNumFrames())]


def get_module_names(thread):
    """
    Returns a sequence of module names from the stack frames of this thread.
    """

    def GetModuleName(i):
        return thread.GetFrameAtIndex(i).GetModule().GetFileSpec().GetFilename()

    return [GetModuleName(i) for i in range(thread.GetNumFrames())]


def get_stack_frames(thread):
    """
    Returns a sequence of stack frames for this thread.
    """

    def GetStackFrame(i):
        return thread.GetFrameAtIndex(i)

    return [GetStackFrame(i) for i in range(thread.GetNumFrames())]


def print_stacktrace(thread, string_buffer=False):
    """Prints a simple stack trace of this thread."""

    output = io.StringIO() if string_buffer else sys.stdout
    target = thread.GetProcess().GetTarget()

    depth = thread.GetNumFrames()

    mods = get_module_names(thread)
    funcs = get_function_names(thread)
    symbols = get_symbol_names(thread)
    files = get_filenames(thread)
    lines = get_line_numbers(thread)
    addrs = get_pc_addresses(thread)

    if thread.GetStopReason() != lldb.eStopReasonInvalid:
        desc = "stop reason=" + stop_reason_to_str(thread.GetStopReason())
    else:
        desc = ""
    print(
        "Stack trace for thread id={0:#x} name={1} queue={2} ".format(
            thread.GetThreadID(), thread.GetName(), thread.GetQueueName()
        )
        + desc,
        file=output,
    )

    for i in range(depth):
        frame = thread.GetFrameAtIndex(i)
        function = frame.GetFunction()

        load_addr = addrs[i].GetLoadAddress(target)
        if not function:
            file_addr = addrs[i].GetFileAddress()
            start_addr = frame.GetSymbol().GetStartAddress().GetFileAddress()
            symbol_offset = file_addr - start_addr
            print(
                "  frame #{num}: {addr:#016x} {mod}`{symbol} + {offset}".format(
                    num=i,
                    addr=load_addr,
                    mod=mods[i],
                    symbol=symbols[i],
                    offset=symbol_offset,
                ),
                file=output,
            )
        else:
            print(
                "  frame #{num}: {addr:#016x} {mod}`{func} at {file}:{line} {args}".format(
                    num=i,
                    addr=load_addr,
                    mod=mods[i],
                    func="%s [inlined]" % funcs[i] if frame.IsInlined() else funcs[i],
                    file=files[i],
                    line=lines[i],
                    args=get_args_as_string(frame, showFuncName=False)
                    if not frame.IsInlined()
                    else "()",
                ),
                file=output,
            )

    if string_buffer:
        return output.getvalue()


def print_stacktraces(process, string_buffer=False):
    """Prints the stack traces of all the threads."""

    output = io.StringIO() if string_buffer else sys.stdout

    print("Stack traces for " + str(process), file=output)

    for thread in process:
        print(print_stacktrace(thread, string_buffer=True), file=output)

    if string_buffer:
        return output.getvalue()


# ===================================
# Utility functions related to Frames
# ===================================


def get_parent_frame(frame):
    """
    Returns the parent frame of the input frame object; None if not available.
    """
    thread = frame.GetThread()
    parent_found = False
    for f in thread:
        if parent_found:
            return f
        if f.GetFrameID() == frame.GetFrameID():
            parent_found = True

    # If we reach here, no parent has been found, return None.
    return None


def get_args_as_string(frame, showFuncName=True):
    """
    Returns the args of the input frame object as a string.
    """
    # arguments     => True
    # locals        => False
    # statics       => False
    # in_scope_only => True
    vars = frame.GetVariables(True, False, False, True)  # type of SBValueList
    args = []  # list of strings
    for var in vars:
        args.append("(%s)%s=%s" % (var.GetTypeName(), var.GetName(), var.GetValue()))
    if frame.GetFunction():
        name = frame.GetFunction().GetName()
    elif frame.GetSymbol():
        name = frame.GetSymbol().GetName()
    else:
        name = ""
    if showFuncName:
        return "%s(%s)" % (name, ", ".join(args))
    else:
        return "(%s)" % (", ".join(args))


def print_registers(frame, string_buffer=False):
    """Prints all the register sets of the frame."""

    output = io.StringIO() if string_buffer else sys.stdout

    print("Register sets for " + str(frame), file=output)

    registerSet = frame.GetRegisters()  # Return type of SBValueList.
    print(
        "Frame registers (size of register set = %d):" % registerSet.GetSize(),
        file=output,
    )
    for value in registerSet:
        # print >> output, value
        print(
            "%s (number of children = %d):" % (value.GetName(), value.GetNumChildren()),
            file=output,
        )
        for child in value:
            print(
                "Name: %s, Value: %s" % (child.GetName(), child.GetValue()), file=output
            )

    if string_buffer:
        return output.getvalue()


def get_registers(frame, kind):
    """Returns the registers given the frame and the kind of registers desired.

    Returns None if there's no such kind.
    """
    registerSet = frame.GetRegisters()  # Return type of SBValueList.
    for value in registerSet:
        if kind.lower() in value.GetName().lower():
            return value

    return None


def get_GPRs(frame):
    """Returns the general purpose registers of the frame as an SBValue.

    The returned SBValue object is iterable.  An example:
        ...
        from lldbutil import get_GPRs
        regs = get_GPRs(frame)
        for reg in regs:
            print("%s => %s" % (reg.GetName(), reg.GetValue()))
        ...
    """
    return get_registers(frame, "general purpose")


def get_FPRs(frame):
    """Returns the floating point registers of the frame as an SBValue.

    The returned SBValue object is iterable.  An example:
        ...
        from lldbutil import get_FPRs
        regs = get_FPRs(frame)
        for reg in regs:
            print("%s => %s" % (reg.GetName(), reg.GetValue()))
        ...
    """
    return get_registers(frame, "floating point")


def get_ESRs(frame):
    """Returns the exception state registers of the frame as an SBValue.

    The returned SBValue object is iterable.  An example:
        ...
        from lldbutil import get_ESRs
        regs = get_ESRs(frame)
        for reg in regs:
            print "%s => %s" % (reg.GetName(), reg.GetValue())
        ...
    """
    return get_registers(frame, "exception state")


# ======================================
# Utility classes/functions for SBValues
# ======================================


class BasicFormatter(object):
    """The basic formatter inspects the value object and prints the value."""

    def format(self, value, buffer=None, indent=0):
        if not buffer:
            output = io.StringIO()
        else:
            output = buffer
        # If there is a summary, it suffices.
        val = value.GetSummary()
        # Otherwise, get the value.
        if val is None:
            val = value.GetValue()
        if val is None and value.GetNumChildren() > 0:
            val = "%s (location)" % value.GetLocation()
        print(
            "{indentation}({type}) {name} = {value}".format(
                indentation=" " * indent,
                type=value.GetTypeName(),
                name=value.GetName(),
                value=val,
            ),
            file=output,
        )
        return output.getvalue()


class ChildVisitingFormatter(BasicFormatter):
    """The child visiting formatter prints the value and its immediate children.

    The constructor takes a keyword arg: indent_child, which defaults to 2.
    """

    def __init__(self, indent_child=2):
        """Default indentation of 2 SPC's for the children."""
        self.cindent = indent_child

    def format(self, value, buffer=None):
        if not buffer:
            output = io.StringIO()
        else:
            output = buffer

        BasicFormatter.format(self, value, buffer=output)
        for child in value:
            BasicFormatter.format(self, child, buffer=output, indent=self.cindent)

        return output.getvalue()


class RecursiveDescentFormatter(BasicFormatter):
    """The recursive descent formatter prints the value and the descendents.

    The constructor takes two keyword args: indent_level, which defaults to 0,
    and indent_child, which defaults to 2.  The current indentation level is
    determined by indent_level, while the immediate children has an additional
    indentation by inden_child.
    """

    def __init__(self, indent_level=0, indent_child=2):
        self.lindent = indent_level
        self.cindent = indent_child

    def format(self, value, buffer=None):
        if not buffer:
            output = io.StringIO()
        else:
            output = buffer
        BasicFormatter.format(self, value, buffer=output, indent=self.lindent)
        new_indent = self.lindent + self.cindent
        for child in value:
            if child.GetSummary() is not None:
                BasicFormatter.format(self, child, buffer=output, indent=new_indent)
            else:
                if child.GetNumChildren() > 0:
                    rdf = RecursiveDescentFormatter(indent_level=new_indent)
                    rdf.format(child, buffer=output)
                else:
                    BasicFormatter.format(self, child, buffer=output, indent=new_indent)

        return output.getvalue()



"""
Test handling of Anonymous Structs, especially that they don't crash lldb.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import os
import shutil
import time


class TestFrameVarAnonStruct(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_frame_var(self):
        self.build()
        self.do_test()

    def do_test(self):
        target = self.createTestTarget()

        # Verify that we don't crash in this case.
        self.expect(
            "target variable 'b.x'",
            error=True,
            substrs=["can't find global variable 'b.x'"],
        )



"""
Test some features of "session history" and history recall.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestHistoryRecall(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_history_recall(self):
        """Test the !N and !-N functionality of the command interpreter."""
        self.do_bang_N_test()

    def test_regex_history(self):
        """Test the regex commands don't add two elements to the history"""
        self.do_regex_history_test()

    def do_regex_history_test(self):
        interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()
        command = "_regexp-break foo.c:12"
        self.runCmd(command, msg="Run the regex break command", inHistory=True)
        interp.HandleCommand("session history", result, True)
        self.assertTrue(result.Succeeded(), "session history ran successfully")
        results = result.GetOutput()
        self.assertIn(command, results, "Recorded the actual command")
        self.assertNotIn(
            "breakpoint set --file 'foo.c' --line 12",
            results,
            "Didn't record the resolved command",
        )

    def do_bang_N_test(self):
        interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()
        interp.HandleCommand("session history", result, True)
        interp.HandleCommand("platform list", result, True)

        interp.HandleCommand("!0", result, False)
        self.assertTrue(
            result.Succeeded(), "!0 command did not work: %s" % (result.GetError())
        )
        self.assertIn(
            "session history", result.GetOutput(), "!0 didn't rerun session history"
        )

        interp.HandleCommand("!-1", result, False)
        self.assertTrue(
            result.Succeeded(), "!-1 command did not work: %s" % (result.GetError())
        )
        self.assertIn("host:", result.GetOutput(), "!-1 didn't rerun platform list.")



"""
Test 'target modules dump separate-debug-info' for oso files.
"""

import json
import os

from lldbsuite.test import lldbtest, lldbutil
from lldbsuite.test.decorators import *


class TestDumpOso(lldbtest.TestBase):
    SHARED_BUILD_TESTCASE = False
    NO_DEBUG_INFO_TESTCASE = True

    def get_osos_from_json_output(self):
        """Returns a dictionary of `symfile` -> {`OSO_PATH` -> oso_info object}."""
        result = {}
        output = json.loads(self.res.GetOutput())
        for symfile_entry in output:
            oso_dict = {}
            for oso_entry in symfile_entry["separate-debug-info-files"]:
                oso_dict[oso_entry["oso_path"]] = oso_entry
            result[symfile_entry["symfile"]] = oso_dict
        return result

    @skipIfRemote
    @skipUnlessDarwin
    def test_shows_oso_loaded_json_output(self):
        self.build(debug_info="dwarf")
        exe = self.getBuildArtifact("a.out")
        main_o = self.getBuildArtifact("main.o")
        foo_o = self.getBuildArtifact("foo.o")

        # Make sure o files exist
        self.assertTrue(os.path.exists(main_o), f'Make sure "{main_o}" file exists')
        self.assertTrue(os.path.exists(foo_o), f'Make sure "{foo_o}" file exists')

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, lldbtest.VALID_TARGET)

        self.runCmd("target modules dump separate-debug-info --json")

        # Check the output
        osos = self.get_osos_from_json_output()
        self.assertTrue(osos[exe][main_o]["loaded"])
        self.assertTrue(osos[exe][foo_o]["loaded"])

    @skipIfRemote
    @skipUnlessDarwin
    def test_shows_oso_not_loaded_json_output(self):
        self.build(debug_info="dwarf")
        exe = self.getBuildArtifact("a.out")
        main_o = self.getBuildArtifact("main.o")
        foo_o = self.getBuildArtifact("foo.o")

        # REMOVE the o files
        os.unlink(main_o)

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, lldbtest.VALID_TARGET)

        self.runCmd("target modules dump separate-debug-info --json")

        # Check the output
        osos = self.get_osos_from_json_output()
        self.assertFalse(osos[exe][main_o]["loaded"])
        self.assertIn("error", osos[exe][main_o])
        self.assertTrue(osos[exe][foo_o]["loaded"])
        self.assertNotIn("error", osos[exe][foo_o])

        # Check with --errors-only
        self.runCmd("target modules dump separate-debug-info --json --errors-only")
        output = self.get_osos_from_json_output()
        self.assertFalse(output[exe][main_o]["loaded"])
        self.assertIn("error", output[exe][main_o])
        self.assertNotIn(foo_o, output[exe])

    @skipIfRemote
    @skipUnlessDarwin
    def test_shows_oso_loaded_table_output(self):
        self.build(debug_info="dwarf")
        exe = self.getBuildArtifact("a.out")
        main_o = self.getBuildArtifact("main.o")
        foo_o = self.getBuildArtifact("foo.o")

        # Make sure o files exist
        self.assertTrue(os.path.exists(main_o), f'Make sure "{main_o}" file exists')
        self.assertTrue(os.path.exists(foo_o), f'Make sure "{foo_o}" file exists')

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, lldbtest.VALID_TARGET)

        self.expect(
            "target modules dump separate-debug-info",
            patterns=[
                r"Symbol file: .*?a\.out",
                'Type: "oso"',
                r"Mod Time\s+Err\s+Oso Path",
                r"0x[a-zA-Z0-9]{16}\s+.*main\.o",
                r"0x[a-zA-Z0-9]{16}\s+.*foo\.o",
            ],
        )

    @skipIfRemote
    @skipUnlessDarwin
    def test_shows_oso_not_loaded_table_output(self):
        self.build(debug_info="dwarf")
        exe = self.getBuildArtifact("a.out")
        main_o = self.getBuildArtifact("main.o")
        foo_o = self.getBuildArtifact("foo.o")

        # REMOVE the o files
        os.unlink(main_o)
        os.unlink(foo_o)

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, lldbtest.VALID_TARGET)

        self.expect(
            "target modules dump separate-debug-info",
            patterns=[
                r"Symbol file: .*?a\.out",
                'Type: "oso"',
                r"Mod Time\s+Err\s+Oso Path",
                r"0x[a-zA-Z0-9]{16}\s+E\s+.*main\.o",
                r"0x[a-zA-Z0-9]{16}\s+E\s+.*foo\.o",
            ],
        )

    @skipIfRemote
    @skipUnlessDarwin
    def test_osos_loaded_symbols_on_demand(self):
        self.build(debug_info="dwarf")
        exe = self.getBuildArtifact("a.out")
        main_o = self.getBuildArtifact("main.o")
        foo_o = self.getBuildArtifact("foo.o")

        # Make sure o files exist
        self.assertTrue(os.path.exists(main_o), f'Make sure "{main_o}" file exists')
        self.assertTrue(os.path.exists(foo_o), f'Make sure "{foo_o}" file exists')

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, lldbtest.VALID_TARGET)

        self.runCmd("target modules dump separate-debug-info --json")

        # Load symbols on-demand
        self.runCmd("settings set symbols.load-on-demand true")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, lldbtest.VALID_TARGET)

        self.runCmd("target modules dump separate-debug-info --json")

        # Check the output
        osos = self.get_osos_from_json_output()
        self.assertTrue(osos[exe][main_o]["loaded"])
        self.assertTrue(osos[exe][foo_o]["loaded"])



from lldbsuite.test.decorators import *
from lldbsuite.test.concurrent_base import ConcurrentEventsBase
from lldbsuite.test.lldbtest import TestBase


@skipIfWindows
class ConcurrentSignalWatchBreak(ConcurrentEventsBase):
    # Atomic sequences are not supported yet for MIPS in LLDB.
    @skipIf(triple="^mips")
    @expectedFailureNetBSD
    @add_test_categories(["watchpoint"])
    def test(self):
        """Test a signal/watchpoint/breakpoint in multiple threads."""
        self.build()
        self.do_thread_actions(
            num_signal_threads=1, num_watchpoint_threads=1, num_breakpoint_threads=1
        )



"""Test Python APIs for process IO."""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ProcessIOTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setup_test(self):
        # Get the full path to our executable to be debugged.
        self.exe = self.getBuildArtifact("process_io")
        self.local_input_file = self.getBuildArtifact("input.txt")
        self.local_output_file = self.getBuildArtifact("output.txt")
        self.local_error_file = self.getBuildArtifact("error.txt")

        self.input_file = os.path.join(
            self.get_process_working_directory(), "input.txt"
        )
        self.output_file = os.path.join(
            self.get_process_working_directory(), "output.txt"
        )
        self.error_file = os.path.join(
            self.get_process_working_directory(), "error.txt"
        )
        self.lines = ["Line 1", "Line 2", "Line 3"]

    @skipIfWindows  # stdio manipulation unsupported on Windows
    @expectedFlakeyLinux(bugnumber="llvm.org/pr26437")
    @skipIfDarwinEmbedded  # I/O redirection like this is not supported on remote iOS devices yet <rdar://problem/54581135>
    def test_stdin_by_api(self):
        """Exercise SBProcess.PutSTDIN()."""
        self.setup_test()
        self.build()
        self.create_target()
        self.run_process(True)
        output = self.process.GetSTDOUT(1000)
        self.check_process_output(output, output)

    @skipIfWindows  # stdio manipulation unsupported on Windows
    @expectedFlakeyLinux(bugnumber="llvm.org/pr26437")
    def test_stdin_redirection(self):
        """Exercise SBLaunchInfo::AddOpenFileAction() for STDIN without specifying STDOUT or STDERR."""
        self.setup_test()
        self.build()
        self.create_target()
        self.redirect_stdin()
        self.run_process(False)
        output = self.process.GetSTDOUT(1000)
        self.check_process_output(output, output)

    @skipIfWindows  # stdio manipulation unsupported on Windows
    @expectedFlakeyLinux(bugnumber="llvm.org/pr26437")
    @skipIfDarwinEmbedded  # debugserver can't create/write files on the device
    def test_stdout_redirection(self):
        """Exercise SBLaunchInfo::AddOpenFileAction() for STDOUT without specifying STDIN or STDERR."""
        self.setup_test()
        self.build()
        self.create_target()
        self.redirect_stdout()
        self.run_process(True)
        output = self.read_output_file_and_delete()
        error = self.process.GetSTDOUT(1000)
        self.check_process_output(output, error)

    @skipIfWindows  # stdio manipulation unsupported on Windows
    @expectedFlakeyLinux(bugnumber="llvm.org/pr26437")
    @skipIfDarwinEmbedded  # debugserver can't create/write files on the device
    def test_stderr_redirection(self):
        """Exercise SBLaunchInfo::AddOpenFileAction() for STDERR without specifying STDIN or STDOUT."""
        self.setup_test()
        self.build()
        self.create_target()
        self.redirect_stderr()
        self.run_process(True)
        output = self.process.GetSTDOUT(1000)
        error = self.read_error_file_and_delete()
        self.check_process_output(output, error)

    @skipIfWindows  # stdio manipulation unsupported on Windows
    @expectedFlakeyLinux(bugnumber="llvm.org/pr26437")
    @skipIfDarwinEmbedded  # debugserver can't create/write files on the device
    def test_stdout_stderr_redirection(self):
        """Exercise SBLaunchInfo::AddOpenFileAction() for STDOUT and STDERR without redirecting STDIN."""
        self.setup_test()
        self.build()
        self.create_target()
        self.redirect_stdout()
        self.redirect_stderr()
        self.run_process(True)
        output = self.read_output_file_and_delete()
        error = self.read_error_file_and_delete()
        self.check_process_output(output, error)

    @skipIfWindows  # stdio manipulation unsupported on Windows
    @expectedFlakeyLinux(bugnumber="llvm.org/pr26437")
    @skipIfDarwinEmbedded  # debugserver can't create/write files on the device
    def test_stdout_stderr_redirection_to_existing_files(self):
        """Exercise SBLaunchInfo::AddOpenFileAction() for STDOUT and STDERR redirect to output files already exist."""
        self.setup_test()
        self.build()
        self.create_target()

        # Create the output and error files with placeholder
        placeholder = "This content should be overwritten."
        # Local file directory and working directory are the same for local debugging
        f = open(self.local_output_file, "w")
        f.write(placeholder)
        f.close()
        f = open(self.local_error_file, "w")
        f.write(placeholder)
        f.close()
        if lldb.remote_platform:
            self.runCmd(
                'platform put-file "{local}" "{remote}"'.format(
                    local=self.local_output_file, remote=self.output_file
                )
            )
            self.runCmd(
                'platform put-file "{local}" "{remote}"'.format(
                    local=self.local_error_file, remote=self.error_file
                )
            )

        self.redirect_stdout()
        self.redirect_stderr()
        self.run_process(True)
        output = self.read_output_file_and_delete()
        error = self.read_error_file_and_delete()
        self.check_process_output(output, error)

    # target_file - path on local file system or remote file system if running remote
    # local_file - path on local system
    def read_file_and_delete(self, target_file, local_file):
        if lldb.remote_platform:
            self.runCmd(
                'platform get-file "{remote}" "{local}"'.format(
                    remote=target_file, local=local_file
                )
            )

        self.assertTrue(
            os.path.exists(local_file),
            'Make sure "{local}" file exists'.format(local=local_file),
        )
        f = open(local_file, "r")
        contents = f.read()
        f.close()

        # TODO: add 'platform delete-file' file command
        # if lldb.remote_platform:
        #    self.runCmd('platform delete-file "{remote}"'.format(remote=target_file))
        os.unlink(local_file)
        return contents

    def read_output_file_and_delete(self):
        return self.read_file_and_delete(self.output_file, self.local_output_file)

    def read_error_file_and_delete(self):
        return self.read_file_and_delete(self.error_file, self.local_error_file)

    def create_target(self):
        """Create the target and launch info that will be used by all tests"""
        self.target = self.dbg.CreateTarget(self.exe)
        self.launch_info = self.target.GetLaunchInfo()
        self.launch_info.SetWorkingDirectory(self.get_process_working_directory())

    def redirect_stdin(self):
        """Redirect STDIN (file descriptor 0) to use our input.txt file

        Make the input.txt file to use when redirecting STDIN, setup a cleanup action
        to delete the input.txt at the end of the test in case exceptions are thrown,
        and redirect STDIN in the launch info."""
        f = open(self.local_input_file, "w")
        for line in self.lines:
            f.write(line + "\n")
        f.close()

        if lldb.remote_platform:
            self.runCmd(
                'platform put-file "{local}" "{remote}"'.format(
                    local=self.local_input_file, remote=self.input_file
                )
            )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            os.unlink(self.local_input_file)
            # TODO: add 'platform delete-file' file command
            # if lldb.remote_platform:
            #    self.runCmd('platform delete-file "{remote}"'.format(remote=self.input_file))

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)
        self.launch_info.AddOpenFileAction(0, self.input_file, True, False)

    def redirect_stdout(self):
        """Redirect STDOUT (file descriptor 1) to use our output.txt file"""
        self.launch_info.AddOpenFileAction(1, self.output_file, False, True)

    def redirect_stderr(self):
        """Redirect STDERR (file descriptor 2) to use our error.txt file"""
        self.launch_info.AddOpenFileAction(2, self.error_file, False, True)

    def run_process(self, put_stdin):
        """Run the process to completion and optionally put lines to STDIN via the API if "put_stdin" is True"""
        # Set the breakpoints
        self.breakpoint = self.target.BreakpointCreateBySourceRegex(
            "Set breakpoint here", lldb.SBFileSpec("main.c")
        )
        self.assertGreater(self.breakpoint.GetNumLocations(), 0, VALID_BREAKPOINT)

        # Launch the process, and do not stop at the entry point.
        error = lldb.SBError()
        # This should launch the process and it should exit by the time we get back
        # because we have synchronous mode enabled
        self.process = self.target.Launch(self.launch_info, error)

        self.assertTrue(error.Success(), "Make sure process launched successfully")
        self.assertTrue(self.process, PROCESS_IS_VALID)

        if self.TraceOn():
            print("process launched.")

        # Frame #0 should be at our breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            self.process, self.breakpoint
        )

        self.assertEqual(len(threads), 1)
        self.thread = threads[0]
        self.frame = self.thread.frames[0]
        self.assertTrue(self.frame, "Frame 0 is valid.")

        if self.TraceOn():
            print("process stopped at breakpoint, sending STDIN via LLDB API.")

        # Write data to stdin via the public API if we were asked to
        if put_stdin:
            for line in self.lines:
                self.process.PutSTDIN(line + "\n")

        # Let process continue so it will exit
        self.process.Continue()
        state = self.process.GetState()
        self.assertState(state, lldb.eStateExited, PROCESS_IS_VALID)

    def check_process_output(self, output, error):
        # Since we launched the process without specifying stdin/out/err,
        # a pseudo terminal is used for stdout/err, and we are satisfied
        # once "input line=>1" appears in stdout.
        # See also main.c.
        if self.TraceOn():
            print("output = '%s'" % output)
            print("error = '%s'" % error)

        for line in self.lines:
            check_line = "input line to stdout: %s" % (line)
            self.assertIn(check_line, output, "verify stdout line shows up in STDOUT")
        for line in self.lines:
            check_line = "input line to stderr: %s" % (line)
            self.assertIn(check_line, error, "verify stderr line shows up in STDERR")



"""
Test the session history command
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SessionHistoryTestCase(TestBase):
    @no_debug_info_test
    def test_history(self):
        self.runCmd("session history --clear", inHistory=False)
        self.runCmd("breakpoint list", check=False, inHistory=True)  # 0
        self.runCmd("register read", check=False, inHistory=True)  # 1
        self.runCmd("apropos hello", check=False, inHistory=True)  # 2
        self.runCmd("memory write", check=False, inHistory=True)  # 3
        self.runCmd("log list", check=False, inHistory=True)  # 4
        self.runCmd("disassemble", check=False, inHistory=True)  # 5
        self.runCmd("expression 1", check=False, inHistory=True)  # 6
        self.runCmd("type summary list -w default", check=False, inHistory=True)  # 7
        self.runCmd("version", check=False, inHistory=True)  # 8
        self.runCmd("frame select 1", check=False, inHistory=True)  # 9

        self.expect(
            "session history -s 3 -c 3",
            inHistory=True,
            substrs=["3: memory write", "4: log list", "5: disassemble"],
        )

        self.expect(
            "session history -s 3 -e 3", inHistory=True, substrs=["3: memory write"]
        )

        self.expect(
            "session history -s 6 -e 7",
            inHistory=True,
            substrs=["6: expression 1", "7: type summary list -w default"],
        )

        self.expect(
            "session history -c 2",
            inHistory=True,
            substrs=["0: breakpoint list", "1: register read"],
        )

        self.expect(
            "session history -e 3 -c 1", inHistory=True, substrs=["3: memory write"]
        )

        self.expect(
            "session history -e 2",
            inHistory=True,
            substrs=["0: breakpoint list", "1: register read", "2: apropos hello"],
        )

        self.expect(
            "session history -s 12",
            inHistory=True,
            substrs=[
                "12: session history -s 6 -e 7",
                "13: session history -c 2",
                "14: session history -e 3 -c 1",
                "15: session history -e 2",
                "16: session history -s 12",
            ],
        )

        self.expect(
            "session history -s end -c 3",
            inHistory=True,
            substrs=[
                "15: session history -e 2",
                "16: session history -s 12",
                "17: session history -s end -c 3",
            ],
        )

        self.expect(
            "session history -s end -e 15",
            inHistory=True,
            substrs=[
                "15: session history -e 2",
                "16: session history -s 12",
                "17: session history -s end -c 3",
                "session history -s end -e 15",
            ],
        )

        self.expect(
            "session history -s 5 -c 1", inHistory=True, substrs=["5: disassemble"]
        )

        self.expect(
            "session history -c 1 -s 5", inHistory=True, substrs=["5: disassemble"]
        )

        self.expect(
            "session history -c 1 -e 3", inHistory=True, substrs=["3: memory write"]
        )

        self.expect(
            "session history -c 1 -e 3 -s 5",
            error=True,
            inHistory=True,
            substrs=[
                "error: --count, --start-index and --end-index cannot be all specified in the same invocation"
            ],
        )



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SmartArrayDataFormatterTestCase(TestBase):
    def test_with_run_command(self):
        """Test data formatter commands."""
        self.build()
        self.data_formatter_commands()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def data_formatter_commands(self):
        """Test that that file and class static variables display correctly."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        process = self.dbg.GetSelectedTarget().GetProcess()

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # check that we are not looping here
        self.runCmd('type summary add --summary-string "${var%V}" SomeData')

        self.expect("frame variable data", substrs=["SomeData @ 0x"])
        # ${var%s}
        self.runCmd('type summary add --summary-string "ptr = ${var%s}" "char *"')

        self.expect("frame variable strptr", substrs=['ptr = "', "Hello world!"])

        self.expect(
            "frame variable other.strptr", substrs=['ptr = "', "Nested Hello world!"]
        )

        self.runCmd(
            'type summary add --summary-string "arr = ${var%s}" -x "char\\[[0-9]+\\]"'
        )

        self.expect("frame variable strarr", substrs=['arr = "', "Hello world!"])

        self.expect(
            "frame variable other.strarr", substrs=['arr = "', "Nested Hello world!"]
        )

        self.expect("expression strarr", substrs=['arr = "', "Hello world!"])

        self.expect(
            "expression other.strarr", substrs=['arr = "', "Nested Hello world!"]
        )

        # ${var%c}
        self.runCmd('type summary add --summary-string "ptr = ${var%c}" "char *"')

        self.expect("frame variable strptr", substrs=['ptr = "', "Hello world!"])

        self.expect(
            "frame variable other.strptr", substrs=['ptr = "', "Nested Hello world!"]
        )

        self.expect("expression strptr", substrs=['ptr = "', "Hello world!"])

        self.expect(
            "expression other.strptr", substrs=['ptr = "', "Nested Hello world!"]
        )

        self.runCmd(
            'type summary add --summary-string "arr = ${var%c}" -x "char\\[[0-9]+\\]"'
        )

        self.expect("frame variable strarr", substrs=['arr = "', "Hello world!"])

        self.expect(
            "frame variable other.strarr", substrs=['arr = "', "Nested Hello world!"]
        )

        self.expect("expression strarr", substrs=['arr = "', "Hello world!"])

        self.expect(
            "expression other.strarr", substrs=['arr = "', "Nested Hello world!"]
        )

        # ${var%char[]}
        self.runCmd(
            'type summary add --summary-string "arr = ${var%char[]}" -x "char\\[[0-9]+\\]"'
        )

        self.expect("frame variable strarr", substrs=['arr = "', "Hello world!"])

        self.expect(
            "frame variable other.strarr", substrs=["arr = ", "Nested Hello world!"]
        )

        self.expect("expression strarr", substrs=['arr = "', "Hello world!"])

        self.expect(
            "expression other.strarr", substrs=["arr = ", "Nested Hello world!"]
        )

        self.runCmd('type summary add --summary-string "ptr = ${var%char[]}" "char *"')

        self.expect("frame variable strptr", substrs=['ptr = "', "Hello world!"])

        self.expect(
            "frame variable other.strptr", substrs=['ptr = "', "Nested Hello world!"]
        )

        self.expect("expression strptr", substrs=['ptr = "', "Hello world!"])

        self.expect(
            "expression other.strptr", substrs=['ptr = "', "Nested Hello world!"]
        )

        # ${var%a}
        self.runCmd(
            'type summary add --summary-string "arr = ${var%a}" -x "char\\[[0-9]+\\]"'
        )

        self.expect("frame variable strarr", substrs=['arr = "', "Hello world!"])

        self.expect(
            "frame variable other.strarr", substrs=["arr = ", "Nested Hello world!"]
        )

        self.expect("expression strarr", substrs=['arr = "', "Hello world!"])

        self.expect(
            "expression other.strarr", substrs=["arr = ", "Nested Hello world!"]
        )

        self.runCmd('type summary add --summary-string "ptr = ${var%a}" "char *"')

        self.expect("frame variable strptr", substrs=['ptr = "', "Hello world!"])

        self.expect(
            "frame variable other.strptr", substrs=['ptr = "', "Nested Hello world!"]
        )

        self.expect("expression strptr", substrs=['ptr = "', "Hello world!"])

        self.expect(
            "expression other.strptr", substrs=['ptr = "', "Nested Hello world!"]
        )

        self.runCmd(
            'type summary add --summary-string "ptr = ${var[]%char[]}" "char *"'
        )

        # I do not know the size of the data, but you are asking for a full array slice..
        # use the ${var%char[]} to obtain a string as result
        self.expect(
            "frame variable strptr", matching=False, substrs=['ptr = "', "Hello world!"]
        )

        self.expect(
            "frame variable other.strptr",
            matching=False,
            substrs=['ptr = "', "Nested Hello world!"],
        )

        self.expect(
            "expression strptr", matching=False, substrs=['ptr = "', "Hello world!"]
        )

        self.expect(
            "expression other.strptr",
            matching=False,
            substrs=['ptr = "', "Nested Hello world!"],
        )

        # You asked an array-style printout...
        self.runCmd(
            'type summary add --summary-string "ptr = ${var[0-1]%char[]}" "char *"'
        )

        self.expect("frame variable strptr", substrs=["ptr = ", "[{H},{e}]"])

        self.expect("frame variable other.strptr", substrs=["ptr = ", "[{N},{e}]"])

        self.expect("expression strptr", substrs=["ptr = ", "[{H},{e}]"])

        self.expect("expression other.strptr", substrs=["ptr = ", "[{N},{e}]"])

        # using [] is required here
        self.runCmd('type summary add --summary-string "arr = ${var%x}" "int [5]"')

        self.expect(
            "frame variable intarr",
            matching=False,
            substrs=["0x00000001,0x00000001,0x00000002,0x00000003,0x00000005"],
        )

        self.expect(
            "frame variable other.intarr",
            matching=False,
            substrs=["0x00000009,0x00000008,0x00000007,0x00000006,0x00000005"],
        )

        self.runCmd('type summary add --summary-string "arr = ${var[]%x}" "int[5]"')

        self.expect(
            "frame variable intarr",
            substrs=[
                "intarr = arr =",
                "0x00000001,0x00000001,0x00000002,0x00000003,0x00000005",
            ],
        )

        self.expect(
            "frame variable other.intarr",
            substrs=[
                "intarr = arr =",
                "0x00000009,0x00000008,0x00000007,0x00000006,0x00000005",
            ],
        )

        # printing each array item as an array
        self.runCmd(
            'type summary add --summary-string "arr = ${var[]%uint32_t[]}" "int[5]"'
        )

        self.expect(
            "frame variable intarr",
            substrs=[
                "intarr = arr =",
                "{0x00000001},{0x00000001},{0x00000002},{0x00000003},{0x00000005}",
            ],
        )

        self.expect(
            "frame variable other.intarr",
            substrs=[
                "intarr = arr = ",
                "{0x00000009},{0x00000008},{0x00000007},{0x00000006},{0x00000005}",
            ],
        )

        # printing full array as an array
        self.runCmd(
            'type summary add --summary-string "arr = ${var%uint32_t[]}" "int[5]"'
        )

        self.expect(
            "frame variable intarr",
            substrs=[
                "intarr = arr =",
                "0x00000001,0x00000001,0x00000002,0x00000003,0x00000005",
            ],
        )

        self.expect(
            "frame variable other.intarr",
            substrs=[
                "intarr = arr =",
                "0x00000009,0x00000008,0x00000007,0x00000006,0x00000005",
            ],
        )

        # printing each array item as an array
        self.runCmd(
            'type summary add --summary-string "arr = ${var[]%float32[]}" "float[7]"'
        )

        self.expect(
            "frame variable flarr",
            substrs=[
                "flarr = arr =",
                "{78.5},{77.25},{78},{76.125},{76.75},{76.875},{77}",
            ],
        )

        self.expect(
            "frame variable other.flarr",
            substrs=[
                "flarr = arr = ",
                "{25.5},{25.25},{25.125},{26.75},{27.375},{27.5},{26.125}",
            ],
        )

        # printing full array as an array
        self.runCmd(
            'type summary add --summary-string "arr = ${var%float32[]}" "float[7]"'
        )

        self.expect(
            "frame variable flarr",
            substrs=["flarr = arr =", "78.5,77.25,78,76.125,76.75,76.875,77"],
        )

        self.expect(
            "frame variable other.flarr",
            substrs=["flarr = arr =", "25.5,25.25,25.125,26.75,27.375,27.5,26.125"],
        )

        # using array smart summary strings for pointers should make no sense
        self.runCmd(
            'type summary add --summary-string "arr = ${var%float32[]}" "float *"'
        )
        self.runCmd(
            'type summary add --summary-string "arr = ${var%int32_t[]}" "int *"'
        )

        self.expect(
            "frame variable flptr",
            matching=False,
            substrs=["78.5,77.25,78,76.125,76.75,76.875,77"],
        )

        self.expect("frame variable intptr", matching=False, substrs=["1,1,2,3,5"])

        # use y and Y
        self.runCmd('type summary add --summary-string "arr = ${var%y}" "float[7]"')
        self.runCmd('type summary add --summary-string "arr = ${var%y}" "int[5]"')

        if process.GetByteOrder() == lldb.eByteOrderLittle:
            self.expect(
                "frame variable flarr",
                substrs=[
                    "flarr = arr =",
                    "00 00 9d 42,00 80 9a 42,00 00 9c 42,00 40 98 42,00 80 99 42,00 c0 99 42,00 00 9a 42",
                ],
            )
        else:
            self.expect(
                "frame variable flarr",
                substrs=[
                    "flarr = arr =",
                    "42 9d 00 00,42 9a 80 00,42 9c 00 00,42 98 40 00,42 99 80 00,42 99 c0 00,42 9a 00 00",
                ],
            )

        if process.GetByteOrder() == lldb.eByteOrderLittle:
            self.expect(
                "frame variable other.flarr",
                substrs=[
                    "flarr = arr =",
                    "00 00 cc 41,00 00 ca 41,00 00 c9 41,00 00 d6 41,00 00 db 41,00 00 dc 41,00 00 d1 41",
                ],
            )
        else:
            self.expect(
                "frame variable other.flarr",
                substrs=[
                    "flarr = arr =",
                    "41 cc 00 00,41 ca 00 00,41 c9 00 00,41 d6 00 00,41 db 00 00,41 dc 00 00,41 d1 00 00",
                ],
            )

        if process.GetByteOrder() == lldb.eByteOrderLittle:
            self.expect(
                "frame variable intarr",
                substrs=[
                    "intarr = arr =",
                    "01 00 00 00,01 00 00 00,02 00 00 00,03 00 00 00,05 00 00 00",
                ],
            )
        else:
            self.expect(
                "frame variable intarr",
                substrs=[
                    "intarr = arr =",
                    "00 00 00 01,00 00 00 01,00 00 00 02,00 00 00 03,00 00 00 05",
                ],
            )

        if process.GetByteOrder() == lldb.eByteOrderLittle:
            self.expect(
                "frame variable other.intarr",
                substrs=[
                    "intarr = arr = ",
                    "09 00 00 00,08 00 00 00,07 00 00 00,06 00 00 00,05 00 00 00",
                ],
            )
        else:
            self.expect(
                "frame variable other.intarr",
                substrs=[
                    "intarr = arr = ",
                    "00 00 00 09,00 00 00 08,00 00 00 07,00 00 00 06,00 00 00 05",
                ],
            )

        self.runCmd('type summary add --summary-string "arr = ${var%Y}" "float[7]"')
        self.runCmd('type summary add --summary-string "arr = ${var%Y}" "int[5]"')

        if process.GetByteOrder() == lldb.eByteOrderLittle:
            self.expect(
                "frame variable flarr",
                substrs=[
                    "flarr = arr =",
                    "00 00 9d 42",
                    "00 80 9a 42",
                    "00 00 9c 42",
                    "00 40 98 42",
                    "00 80 99 42",
                    "00 c0 99 42",
                    "00 00 9a 42",
                ],
            )
        else:
            self.expect(
                "frame variable flarr",
                substrs=[
                    "flarr = arr =",
                    "42 9d 00 00",
                    "42 9a 80 00",
                    "42 9c 00 00",
                    "42 98 40 00",
                    "42 99 80 00",
                    "42 99 c0 00",
                    "42 9a 00 00",
                ],
            )

        if process.GetByteOrder() == lldb.eByteOrderLittle:
            self.expect(
                "frame variable other.flarr",
                substrs=[
                    "flarr = arr =",
                    "00 00 cc 41",
                    "00 00 ca 41",
                    "00 00 c9 41",
                    "00 00 d6 41",
                    "00 00 db 41",
                    "00 00 dc 41",
                    "00 00 d1 41",
                ],
            )
        else:
            self.expect(
                "frame variable other.flarr",
                substrs=[
                    "flarr = arr =",
                    "41 cc 00 00",
                    "41 ca 00 00",
                    "41 c9 00 00",
                    "41 d6 00 00",
                    "41 db 00 00",
                    "41 dc 00 00",
                    "41 d1 00 00",
                ],
            )

        if process.GetByteOrder() == lldb.eByteOrderLittle:
            self.expect(
                "frame variable intarr",
                substrs=["intarr = arr =", "....,01 00 00 00", "....,05 00 00 00"],
            )
        else:
            self.expect(
                "frame variable intarr",
                substrs=["intarr = arr =", "....,00 00 00 01", "....,00 00 00 05"],
            )

        if process.GetByteOrder() == lldb.eByteOrderLittle:
            self.expect(
                "frame variable other.intarr",
                substrs=["intarr = arr = ", "09 00 00 00", "....,07 00 00 00"],
            )
        else:
            self.expect(
                "frame variable other.intarr",
                substrs=["intarr = arr = ", "00 00 00 09", "....,00 00 00 07"],
            )



# coding=utf8
"""
Test std::u8string_view summary.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class StdU8StringViewDataFormatterTestCase(TestBase):
    TEST_WITH_PDB_DEBUG_INFO = True
    SHARED_BUILD_TESTCASE = False

    def do_test(self):
        lldbutil.run_to_source_breakpoint(
            self, "Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        string_view_name = (
            "std::basic_string_view<char8_t, std::char_traits<char8_t>>"
            if self.getDebugInfo() == "pdb"
            else "std::u8string_view"
        )

        self.expect(
            "frame variable",
            substrs=[
                f'({string_view_name}) u8_string_small = u8"🍄"',
                f'({string_view_name}) u8_string = u8"❤️👍📄📁😃🧑‍🌾"',
                f'({string_view_name}) u8_empty = u8""',
                f'({string_view_name}) u8_text = u8"ABCd"',
            ],
        )

    @expectedFailureAll(bugnumber="No libc++ formatters for std::u8string_view yet.")
    @add_test_categories(["libc++"])
    def test_libcxx(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test()

    @add_test_categories(["libstdcxx"])
    def test_libstdcxx(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test()

    @add_test_categories(["msvcstl"])
    def test_msvc(self):
        self.build()
        self.do_test()



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCaseTypedefToOuterFwd(TestBase):
    """
    We find a global variable whose type is forward declared
    (whose definition is in either main.o or lib.o). We then
    try to get the 'Ref' typedef nested within that forward
    declared type. This test makes sure we correctly resolve
    this typedef.

    We test this for two cases, where the definition lives
    in main.o or lib.o.
    """

    def check_global_var(self, target, name: str):
        var = target.FindFirstGlobalVariable(name)
        self.assertSuccess(var.GetError(), f"Found {name}")

        var_type = var.GetType()
        self.assertTrue(var_type)

        impl = var_type.GetPointeeType()
        self.assertTrue(impl)

        ref = impl.FindDirectNestedType("Ref")
        self.assertTrue(ref)

        self.assertEqual(ref.GetCanonicalType(), var_type)

    def test(self):
        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.check_global_var(target, "gLibExternalDef")
        self.check_global_var(target, "gMainExternalDef")



"""
Ensure that when the interrupt is raised we still make frame 0.
and make sure "GetNumFrames" isn't interrupted.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestInterruptingBacktrace(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @expectedFailureAll(oslist=["windows"], archs=["aarch64"])
    @skipIf(oslist=["linux"], archs=["arm$"])
    def test_backtrace_interrupt(self):
        """Use RequestInterrupt followed by stack operations
        to ensure correct interrupt behavior for stacks."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.bt_interrupt_test()

    def bt_interrupt_test(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        # Now continue, and when we stop we will have crashed.
        process.Continue()
        self.dbg.RequestInterrupt()

        # Be sure to turn this off again:
        def cleanup():
            if self.dbg.InterruptRequested():
                self.dbg.CancelInterruptRequest()

        self.addTearDownHook(cleanup)

        frame_0 = thread.GetFrameAtIndex(0)
        self.assertTrue(frame_0.IsValid(), "Got a good 0th frame")
        # The interrupt flag is up already, so any attempt to backtrace
        # should be cut short:
        frame_1 = thread.GetFrameAtIndex(1)
        self.assertFalse(frame_1.IsValid(), "Prevented from getting more frames")
        # Since GetNumFrames is a contract, we don't interrupt it:
        num_frames = thread.GetNumFrames()
        print(f"Number of frames: {num_frames}")
        self.assertGreater(num_frames, 1, "Got many frames")

        self.dbg.CancelInterruptRequest()



"""
Test lldb data formatter subsystem for std::span
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class StdSpanDataFormatterTestCase(TestBase):
    TEST_WITH_PDB_DEBUG_INFO = True
    SHARED_BUILD_TESTCASE = False

    def findVariable(self, name):
        var = self.frame().FindVariable(name)
        self.assertTrue(var.IsValid())
        return var

    def check_size(self, var_name, size):
        var = self.findVariable(var_name)
        self.assertEqual(var.GetNumChildren(), size)

    def check_numbers(self, var_name):
        """Helper to check that data formatter sees contents of std::span correctly"""

        expectedSize = 5
        self.check_size(var_name, expectedSize)

        self.expect_expr(
            var_name,
            result_type=f"std::span<int, {expectedSize}>",
            result_summary=f"size={expectedSize}",
            result_children=[
                ValueCheck(name="[0]", value="1"),
                ValueCheck(name="[1]", value="12"),
                ValueCheck(name="[2]", value="123"),
                ValueCheck(name="[3]", value="1234"),
                ValueCheck(name="[4]", value="12345"),
            ],
        )

        # check access-by-index
        self.expect_var_path(f"{var_name}[0]", type="int", value="1")
        self.expect_var_path(f"{var_name}[1]", type="int", value="12")
        self.expect_var_path(f"{var_name}[2]", type="int", value="123")
        self.expect_var_path(f"{var_name}[3]", type="int", value="1234")
        self.expect_var_path(f"{var_name}[4]", type="int", value="12345")

    def do_test(self):
        """Test that std::span variables are formatted correctly when printed."""
        (self.target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.cpp", False)
        )

        lldbutil.continue_to_breakpoint(process, bkpt)

        # std::span of std::array with extents known at compile-time
        self.check_numbers("numbers_span")

        # check access to synthetic children for static spans
        self.runCmd(
            'type summary add --summary-string "item 0 is ${var[0]}" -x "std::span<" span'
        )
        self.expect_expr(
            "numbers_span",
            result_type="std::span<int, 5>",
            result_summary="item 0 is 1",
        )

        self.runCmd(
            'type summary add --summary-string "item 0 is ${svar[0]}" -x "std::span<" span'
        )
        self.expect_expr(
            "numbers_span",
            result_type="std::span<int, 5>",
            result_summary="item 0 is 1",
        )

        self.runCmd("type summary clear")

        # New span with strings
        lldbutil.continue_to_breakpoint(process, bkpt)

        expectedStringSpanChildren = [
            ValueCheck(name="[0]", summary='"smart"'),
            ValueCheck(name="[1]", summary='"!!!"'),
        ]

        self.expect_var_path(
            "strings_span", summary="size=2", children=expectedStringSpanChildren
        )

        # check access to synthetic children for dynamic spans
        dynamic_string_span = (
            "std::span<std::basic_string<char, std::char_traits<char>, std::allocator<char>>, -1>"
            if self.getDebugInfo() == "pdb"
            else "dynamic_string_span"
        )
        self.runCmd(
            f'type summary add --summary-string "item 0 is ${{var[0]}}" "{dynamic_string_span}"'
        )
        self.expect_var_path("strings_span", summary='item 0 is "smart"')

        self.runCmd(
            f'type summary add --summary-string "item 0 is ${{svar[0]}}" "{dynamic_string_span}"'
        )
        self.expect_var_path("strings_span", summary='item 0 is "smart"')

        self.runCmd(f'type summary delete "{dynamic_string_span}"')

        # test summaries based on synthetic children
        self.runCmd(
            f'type summary add --summary-string "span has ${{svar%#}} items" -e "{dynamic_string_span}"'
        )

        self.expect_var_path("strings_span", summary="span has 2 items")

        self.expect_var_path(
            "strings_span",
            summary="span has 2 items",
            children=expectedStringSpanChildren,
        )

        # check access-by-index
        self.expect_var_path("strings_span[0]", summary='"smart"')
        self.expect_var_path("strings_span[1]", summary='"!!!"')

        # Newly inserted value not visible to span
        lldbutil.continue_to_breakpoint(process, bkpt)

        self.expect_expr(
            "strings_span",
            result_summary="span has 2 items",
            result_children=expectedStringSpanChildren,
        )

        self.runCmd(f'type summary delete "{dynamic_string_span}"')

        lldbutil.continue_to_breakpoint(process, bkpt)

        # Empty spans
        self.expect_expr(
            "static_zero_span", result_type="std::span<int, 0>", result_summary="size=0"
        )
        self.check_size("static_zero_span", 0)

        self.expect_expr("dynamic_zero_span", result_summary="size=0")
        self.check_size("dynamic_zero_span", 0)

        # Nested spans
        self.expect_expr(
            "nested",
            result_summary="size=2",
            result_children=[
                ValueCheck(
                    name="[0]", summary="size=2", children=expectedStringSpanChildren
                ),
                ValueCheck(
                    name="[1]", summary="size=2", children=expectedStringSpanChildren
                ),
            ],
        )
        self.check_size("nested", 2)

    def do_test_ref_and_ptr(self):
        """Test that std::span is correctly formatted when passed by ref and ptr"""
        (self.target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Stop here to check by ref", lldb.SBFileSpec("main.cpp", False)
        )

        # The reference should display the same was as the value did
        self.check_numbers("ref")

        # The pointer should just show the right number of elements:

        self.expect("frame variable ptr", patterns=["ptr = 0x[0-9a-f]+ size=5"])

    @skipIf(compiler="clang", compiler_version=["<", "11.0"])
    @add_test_categories(["libc++"])
    def test_libcxx(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test()

    @skipIf(compiler="clang", compiler_version=["<", "11.0"])
    @add_test_categories(["libc++"])
    def test_ref_and_ptr_libcxx(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test_ref_and_ptr()

    @skipIf(compiler="clang", compiler_version=["<", "11.0"])
    @add_test_categories(["libstdcxx"])
    def test_libstdcxx(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test()

    @skipIf(compiler="clang", compiler_version=["<", "11.0"])
    @add_test_categories(["libstdcxx"])
    def test_ref_and_ptr_libstdcxx(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test_ref_and_ptr()

    @add_test_categories(["msvcstl"])
    def test_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test()

    @add_test_categories(["msvcstl"])
    def test_ref_and_ptr_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test_ref_and_ptr()



import lldb


def foo_SummaryProvider(valobj, dict):
    a = valobj.GetChildMemberWithName("a")
    a_ptr = valobj.GetChildMemberWithName("a_ptr")
    bar = valobj.GetChildMemberWithName("b")
    i = bar.GetChildMemberWithName("i")
    i_ptr = bar.GetChildMemberWithName("i_ptr")
    b_ref = bar.GetChildMemberWithName("b_ref")
    b_ref_ptr = b_ref.AddressOf()
    b_ref = b_ref_ptr.Dereference()
    h = b_ref.GetChildMemberWithName("h")
    k = b_ref.GetChildMemberWithName("k")
    return (
        "a = "
        + str(a.GetValueAsUnsigned(0))
        + ", a_ptr = "
        + str(a_ptr.GetValueAsUnsigned(0))
        + " -> "
        + str(a_ptr.Dereference().GetValueAsUnsigned(0))
        + ", i = "
        + str(i.GetValueAsUnsigned(0))
        + ", i_ptr = "
        + str(i_ptr.GetValueAsUnsigned(0))
        + " -> "
        + str(i_ptr.Dereference().GetValueAsUnsigned(0))
        + ", b_ref = "
        + str(b_ref.GetValueAsUnsigned(0))
        + ", h = "
        + str(h.GetValueAsUnsigned(0))
        + " , k = "
        + str(k.GetValueAsUnsigned(0))
    )


def foo_SummaryProvider3(valobj, dict, options):
    if not isinstance(options, lldb.SBTypeSummaryOptions):
        raise Exception()
    return foo_SummaryProvider(valobj, dict) + ", WITH_OPTS"



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

import gc
import os


class ReplaceDllTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    @skipUnlessWindows
    def test(self):
        """
        Test that LLDB unlocks module files once all references are released.
        """

        exe = self.getBuildArtifact("a.out")
        foo = self.getBuildArtifact("foo.dll")
        bar = self.getBuildArtifact("bar.dll")

        self.build(
            dictionary={
                "DYLIB_NAME": "foo",
                "DYLIB_C_SOURCES": "foo.c",
                "C_SOURCES": "test.c",
            }
        )
        self.build(
            dictionary={
                "DYLIB_ONLY": "YES",
                "DYLIB_NAME": "bar",
                "DYLIB_C_SOURCES": "bar.c",
            }
        )

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        shlib_names = ["foo"]
        environment = self.registerSharedLibrariesWithTarget(target, shlib_names)
        process = target.LaunchSimple(
            None, environment, self.get_process_working_directory()
        )
        self.assertEqual(process.GetExitStatus(), 42)

        module = next((m for m in target.modules if "foo" in m.file.basename), None)
        self.assertIsNotNone(module)
        self.assertEqual(module.file.fullpath, foo)

        target.RemoveModule(module)
        del module
        gc.collect()

        self.dbg.MemoryPressureDetected()

        os.remove(foo)
        os.rename(bar, foo)

        process = target.LaunchSimple(
            None, environment, self.get_process_working_directory()
        )
        self.assertEqual(process.GetExitStatus(), 43)



"""Test that lldb recognizes enum variant emitted by Rust compiler """
import logging

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from RustEnumValue import RustEnumValue


class TestRustEnumStructs(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        src_dir = self.getSourceDir()
        yaml_path = os.path.join(src_dir, "main.yaml")
        obj_path = self.getBuildArtifact("main.o")
        self.yaml2obj(yaml_path, obj_path)
        self.dbg.CreateTarget(obj_path)

    def getFromGlobal(self, name):
        values = self.target().FindGlobalVariables(name, 1)
        self.assertEqual(values.GetSize(), 1)
        return RustEnumValue(values[0])

    def test_enum_instance(self):
        # static ENUM_INSTANCE: A = A::A(B::B(10));
        value = self.getFromGlobal("ENUM_INSTANCE").getCurrentValue()
        self.assertEqual(value.GetType().GetDisplayTypeName(), "main::A::A")

        value_b = RustEnumValue(value.GetChildAtIndex(0))
        self.assertEqual(
            value_b.getCurrentValue()
            .GetChildAtIndex(0)
            .GetData()
            .GetUnsignedInt8(lldb.SBError(), 0),
            10,
        )
